@@ -7,11 +7,15 @@ Unit and system tests for tahoe-fuse.
 # operation fails, because this does not indicate a fuse interface
 # failure.
 
+# TODO: Unmount after tests regardless of failure or success!
+
 # TODO: Test mismatches between tahoe and fuse/posix.  What about nodes
 # with crazy names ('\0', unicode, '/', '..')?  Huuuuge files?
 # Huuuuge directories...  As tahoe approaches production quality, it'd
 # be nice if the fuse interface did so also by hardening against such cases.
 
+# FIXME: Only create / launch necessary nodes.  Do we still need an introducer and three nodes?
+
 # FIXME: This framework might be replaceable with twisted.trial,
 # especially the "layer" design, which is a bit cumbersome when
 # using recursion to manage multiple clients.
@@ -23,16 +27,26 @@ import sys, os, shutil, unittest, subprocess
 import tempfile, re, time, signal, random, httplib
 import traceback
 
-import tahoe_fuse
+# Import fuse implementations:
+FuseDir = os.path.join('.', 'contrib', 'fuse')
+if not os.path.isdir(FuseDir):
+    raise SystemExit('''
+Could not find directory "%s".  Please run this script from the tahoe
+source base directory.
+''' % (FuseDir,))
+
+sys.path.append(os.path.join(FuseDir, 'impl_a'))
+import tahoe_fuse as impl_a
+
+sys.path.append(os.path.join(FuseDir, 'impl_b'))
+import pyfuse.tahoe as impl_b
 
 
 ### Main flow control:
-def main(args = sys.argv[1:]):
+def main(args = sys.argv):
     target = 'all'
-    if args:
-        if len(args) != 1:
-            raise SystemExit(Usage)
-        target = args[0]
+    if len(args) > 1:
+        target = args.pop(1)
 
     if target not in ('all', 'unit', 'system'):
         raise SystemExit(Usage)
@@ -86,26 +100,18 @@ class SystemTest (object):
         self.fullcleanup = fullcleanup
         print '\n*** Setting up system tests.'
         try:
-            failures, total = self.init_cli_layer()
-            print '\n*** System Tests complete: %d failed out of %d.' % (failures, total)           
-        except self.SetupFailure, sfail:
+            results = self.init_cli_layer()
+            print '\n*** System Tests complete:'
+            for result in results:
+                print 'Implementation %s: %d failed out of %d.' % result           
+        except SetupFailure, sfail:
             print
             print sfail
             print '\n*** System Tests were not successfully completed.' 
 
     def init_cli_layer(self):
         '''This layer finds the appropriate tahoe executable.'''
-        runtestpath = os.path.abspath(sys.argv[0])
-        path = runtestpath
-        for expectedname in ('runtests.py', 'fuse', 'contrib'):
-            path, name = os.path.split(path)
-
-            if name != expectedname:
-                reason = 'Unexpected test script path: %r\n'
-                reason += 'The system test script must be run from the source directory.'
-                raise self.SetupFailure(reason, runtestpath)
-
-        self.cliexec = os.path.join(path, 'bin', 'tahoe')
+        self.cliexec = os.path.join('.', 'bin', 'tahoe')
         version = self.run_tahoe('--version')
         print 'Using %r with version:\n%s' % (self.cliexec, version.rstrip())
 
@@ -149,8 +155,8 @@ class SystemTest (object):
             self.stop_node(introbase)
         
     TotalClientsNeeded = 3
-    def launch_clients_layer(self, introbase, clientnum = 1):
-        if clientnum > self.TotalClientsNeeded:
+    def launch_clients_layer(self, introbase, clientnum = 0):
+        if clientnum >= self.TotalClientsNeeded:
             return self.create_test_dirnode_layer()
 
         tmpl = 'Launching client %d of %d.'
@@ -162,18 +168,23 @@ class SystemTest (object):
         output = self.run_tahoe('create-client', '--basedir', base)
         self.check_tahoe_output(output, ExpectedCreationOutput, base)
 
-        if clientnum == 1:
+        webportpath = os.path.join(base, 'webport')
+        if clientnum == 0:
             # The first client is special:
             self.clientbase = base
             self.port = random.randrange(1024, 2**15)
 
-            f = open(os.path.join(base, 'webport'), 'w')
+            f = open(webportpath, 'w')
             f.write('tcp:%d:interface=127.0.0.1\n' % self.port)
             f.close()
+        else:
+            os.remove(webportpath)
+            
 
         introfurl = os.path.join(introbase, 'introducer.furl')
 
-        self.polling_operation(lambda : os.path.isfile(introfurl))
+        self.polling_operation(lambda : os.path.isfile(introfurl),
+                               'introducer.furl creation')
         shutil.copy(introfurl, base)
 
         # NOTE: We assume if tahoe exist with non-zero status, no separate
@@ -199,68 +210,64 @@ class SystemTest (object):
 
         return self.mount_fuse_layer(cap)
         
-    def mount_fuse_layer(self, fusebasecap, fusepause=2.0):
-        print 'Mounting fuse interface.'
+    def mount_fuse_layer(self, fusebasecap):
+        mpbase = os.path.join(self.testroot, 'mountpoint')
+        os.mkdir(mpbase)
 
-        mp = os.path.join(self.testroot, 'mountpoint')
-        os.mkdir(mp)
+        results = []
 
-        thispath = os.path.abspath(sys.argv[0])
-        thisdir = os.path.dirname(thispath)
-        fusescript = os.path.join(thisdir, 'tahoe_fuse.py')
-        try:
-            proc = subprocess.Popen([fusescript,
-                                     mp,
-                                     '-f',
-                                     '--basedir', self.clientbase])
+        # Mount and test each implementation:
+        for implnum, implmanklass in enumerate([Impl_A_ProcessManager, Impl_B_ProcessManager]):
+            implman = implmanklass(self.clientbase, mpbase)
+            print '\n*** Testing impl #%d: %r' % (implnum, implman.Name)
 
-            # The mount is verified by the test_layer, but we sleep to
-            # avoid race conditions against the first few tests.
-            time.sleep(fusepause)
+            implman.setup()
 
-            return self.run_test_layer(fusebasecap, mp)
-                
-        finally:
-            print '\n*** Cleaning up system test'
+            try:
+                failures, total = self.run_test_layer(fusebasecap, implman)
+                result = (implman.Name, failures, total)
+                tmpl = '\n*** Test Results implementation %s: %d failed out of %d.'
+                print tmpl % result
+                results.append(result)
 
-            if proc.poll() is None:
-                print 'Killing fuse interface.'
-                os.kill(proc.pid, signal.SIGTERM)
-                print 'Waiting for the fuse interface to exit.'
-                proc.wait()
-            
-    def run_test_layer(self, fbcap, mountpoint):
-        total = failures = 0
-        for name in sorted(dir(self)):
-            if name.startswith('test_'):
-                total += 1
-                print '\n*** Running test #%d: %s' % (total, name)
-                try:
-                    testcap = self.create_dirnode()
-                    self.attach_node(fbcap, testcap, name)
+            finally:
+                implman.cleanup()
+
+        return results
+
+    def run_test_layer(self, fbcap, iman):
+        testnames = [n for n in sorted(dir(self)) if n.startswith('test_')]
+
+        failures = 0
+        for testnum, testname in enumerate(testnames):
+            print '\n*** Running test #%d: %s' % (testnum, testname)
+            try:
+                testcap = self.create_dirnode()
+                dirname = '%s_%s' % (iman.Name, testname)
+                self.attach_node(fbcap, testcap, dirname)
                     
-                    method = getattr(self, name)
-                    method(testcap, testdir = os.path.join(mountpoint, name))
-                    print 'Test succeeded.'
-                except self.TestFailure, f:
-                    print f
-                    failures += 1
-                except:
-                    print 'Error in test code...  Cleaning up.'
-                    raise
+                method = getattr(self, testname)
+                method(testcap, testdir = os.path.join(iman.mountpath, dirname))
+                print 'Test succeeded.'
+            except TestFailure, f:
+                print f
+                failures += 1
+            except:
+                print 'Error in test code...  Cleaning up.'
+                raise
 
-        return (failures, total)
+        return (failures, len(testnames))
 
 
     # Tests:
     def test_directory_existence(self, testcap, testdir):
-        if not os.path.isdir(testdir):
-            raise self.TestFailure('Attached test directory not found: %r', testdir)
+        if not wrap_os_error(os.path.isdir, testdir):
+            raise TestFailure('Attached test directory not found: %r', testdir)
             
     def test_empty_directory_listing(self, testcap, testdir):
-        listing = os.listdir(testdir)
+        listing = wrap_os_error(os.listdir, testdir)
         if listing:
-            raise self.TestFailure('Expected empty directory, found: %r', listing)
+            raise TestFailure('Expected empty directory, found: %r', listing)
     
     def test_directory_listing(self, testcap, testdir):
         names = []
@@ -283,17 +290,18 @@ class SystemTest (object):
 
         names.sort()
             
-        listing = os.listdir(testdir)
+        listing = wrap_os_error(os.listdir, testdir)
         listing.sort()
+
         if listing != names:
             tmpl = 'Expected directory list containing %r but fuse gave %r'
-            raise self.TestFailure(tmpl, names, listing)
+            raise TestFailure(tmpl, names, listing)
 
         for file, size in filesizes.items():
-            st = os.stat(os.path.join(testdir, file))
+            st = wrap_os_error(os.stat, os.path.join(testdir, file))
             if st.st_size != size:
                 tmpl = 'Expected %r size of %r but fuse returned %r'
-                raise self.TestFailure(tmpl, file, size, st.st_size)
+                raise TestFailure(tmpl, file, size, st.st_size)
     
     def test_file_contents(self, testcap, testdir):
         name = 'hw.txt'
@@ -307,11 +315,11 @@ class SystemTest (object):
             found = open(path, 'r').read()
         except Exception, err:
             tmpl = 'Could not read file contents of %r: %r'
-            raise self.TestFailure(tmpl, path, err)
+            raise TestFailure(tmpl, path, err)
 
         if found != body:
             tmpl = 'Expected file contents %r but found %r'
-            raise self.TestFailure(tmpl, body, found)
+            raise TestFailure(tmpl, body, found)
         
             
     # Utilities:
@@ -324,7 +332,7 @@ class SystemTest (object):
             tmpl += 'Command arguments: %r\n'
             tmpl += 'Exit status: %r\n'
             tmpl += 'Output:\n%s\n[End of tahoe output.]\n'
-            raise self.SetupFailure(tmpl,
+            raise SetupFailure(tmpl,
                                     self.cliexec,
                                     realargs,
                                     status,
@@ -368,7 +376,7 @@ class SystemTest (object):
             tmpl += 'Request: %r %r\n'
             tmpl += 'Body:\n%s\n'
             tmpl += 'Response:\nStatus %r\nBody:\n%s'
-            raise self.SetupFailure(tmpl,
+            raise SetupFailure(tmpl,
                                     method, path,
                                     body or '',
                                     resp.status, body)
@@ -386,7 +394,7 @@ class SystemTest (object):
                                 replace = 'false')
         assert body.strip() == childcap, `status, dircap, childcap, childname`
 
-    def polling_operation(self, operation, timeout = 10.0, pollinterval = 0.2):
+    def polling_operation(self, operation, polldesc, timeout = 10.0, pollinterval = 0.2):
         totaltime = timeout # Fudging for edge-case SetupFailure description...
         
         totalattempts = int(timeout / pollinterval)
@@ -420,29 +428,30 @@ class SystemTest (object):
                 #print tmpl % (attempt+1, opdelay, realinterval)
                 time.sleep(realinterval)
 
-        tmpl = 'Timeout after waiting for creation of introducer.furl.\n'
+                
+        tmpl = 'Timeout while polling for: %s\n'
         tmpl += 'Waited %.2f seconds (%d polls).'
-        raise self.SetupFailure(tmpl, totaltime, attempt+1)
+        raise SetupFailure(tmpl, polldesc, totaltime, attempt+1)
 
     def warn(self, tmpl, *args):
         print ('Test Warning: ' + tmpl) % args
 
 
-    # SystemTest Exceptions:
-    class Failure (Exception):
-        def __init__(self, tmpl, *args):
-            msg = self.Prefix + (tmpl % args)
-            Exception.__init__(self, msg)
+# SystemTest Exceptions:
+class Failure (Exception):
+    def __init__(self, tmpl, *args):
+        msg = self.Prefix + (tmpl % args)
+        Exception.__init__(self, msg)
     
-    class SetupFailure (Failure):
-        Prefix = 'Setup Failure - The test framework encountered an error:\n'
+class SetupFailure (Failure):
+    Prefix = 'Setup Failure - The test framework encountered an error:\n'
 
-    class TestFailure (Failure):
-        Prefix = 'TestFailure: '
+class TestFailure (Failure):
+    Prefix = 'TestFailure: '
             
 
 ### Unit Tests:
-class TestUtilFunctions (unittest.TestCase):
+class Impl_A_UnitTests (unittest.TestCase):
     '''Tests small stand-alone functions.'''
     def test_canonicalize_cap(self):
         iopairs = [('http://127.0.0.1:8123/uri/URI:DIR2:yar9nnzsho6czczieeesc65sry:upp1pmypwxits3w9izkszgo1zbdnsyk3nm6h7e19s7os7s6yhh9y',
@@ -451,12 +460,73 @@ class TestUtilFunctions (unittest.TestCase):
                     'URI:CHK:k7ktp1qr7szmt98s1y3ha61d9w:8tiy8drttp65u79pjn7hs31po83e514zifdejidyeo1ee8nsqfyy:3:12:242?filename=welcome.html')]
 
         for input, output in iopairs:
-            result = tahoe_fuse.canonicalize_cap(input)
+            result = impl_a.canonicalize_cap(input)
             self.failUnlessEqual(output, result, 'input == %r' % (input,))
                     
 
 
 ### Misc:
+class ImplProcessManager (object):
+    '''Subclasses must have Name and Mod class attributes.'''
+    def __init__(self, clientbase, mpbase):
+        self.clientbase = clientbase
+        self.mountpath = os.path.join(mpbase, self.Name)
+        self.script = self.Mod.__file__
+        os.mkdir(self.mountpath)
+
+    
+class Impl_A_ProcessManager (ImplProcessManager):
+    Name = 'impl_a'
+    Mod = impl_a
+    
+    def setup(self):
+        print 'Mounting implementation: %s' % (self.Name,)
+        exitcode, output = gather_output(['python',
+                                          self.script,
+                                          self.mountpath,
+                                          '--basedir', self.clientbase])
+
+        if exitcode != 0 or output:
+            tmpl = '%r failed to launch:\n'
+            tmpl += 'Exit Status: %r\n'
+            tmpl += 'Output:\n%s\n'
+            raise SetupFailure(tmpl, implpath, exitcode, output)
+
+    def cleanup(self):
+        print 'Unmounting implementation: %s' % (self.Name,)
+        args = ['fusermount', '-u', self.mountpath]
+        ec, out = gather_output(args)
+        if ec != 0 or out:
+            tmpl = 'fusermount failed to unmount:\n'
+            tmpl += 'Arguments: %r\n'
+            tmpl += 'Exit Status: %r\n'
+            tmpl += 'Output:\n%s\n'
+            raise SetupFailure(tmpl, args, ec, out)
+
+        
+class Impl_B_ProcessManager (ImplProcessManager):
+    Name = 'impl_b'
+    Mod = impl_b
+    
+    def setup(self):
+        print 'Mounting implementation: %s' % (self.Name,)
+        self.proc = subprocess.Popen(['python',
+                                      self.script,
+                                      self.mountpath,
+                                      '--basedir', self.clientbase])
+
+    def cleanup(self):
+        print 'Unmounting implementation: %s' % (self.Name,)
+        args = ['fusermount', '-u', self.mountpath]
+        ec, out = gather_output(args)
+        if ec != 0 or out:
+            tmpl = 'fusermount failed to unmount:\n'
+            tmpl += 'Arguments: %r\n'
+            tmpl += 'Exit Status: %r\n'
+            tmpl += 'Output:\n%s\n'
+            raise SetupFailure(tmpl, args, ec, out)
+
+        
 def gather_output(*args, **kwargs):
     '''
     This expects the child does not require input and that it closes
@@ -471,6 +541,13 @@ def gather_output(*args, **kwargs):
     return (exitcode, output)
     
 
+def wrap_os_error(meth, *args):
+    try:
+        return meth(*args)
+    except os.error, e:
+        raise TestFailure('%s', e)
+
+
 ExpectedCreationOutput = r'(introducer|client) created in (?P<path>.*?)\n'
 ExpectedStartOutput = r'STARTING (?P<path>.*?)\n(introducer|client) node probably started'
 
