@@ -8,7 +8,8 @@ from allmydata.mutable.common import NotMutableError
 from allmydata.mutable.node import MutableFileNode
 from allmydata.interfaces import IMutableFileNode, IDirectoryNode,\
      IURI, IFileNode, IMutableFileURI, IVerifierURI, IFilesystemNode, \
-     ExistingChildError
+     ExistingChildError, ICheckable
+from allmydata.immutable.checker import DeepCheckResults
 from allmydata.util import hashutil, mathutil
 from allmydata.util.hashutil import netstring
 from allmydata.util.limiter import ConcurrencyLimiter
@@ -112,7 +113,7 @@ class Adder:
         return new_contents
 
 class NewDirectoryNode:
-    implements(IDirectoryNode)
+    implements(IDirectoryNode, ICheckable)
     filenode_class = MutableFileNode
 
     def __init__(self, client):
@@ -242,9 +243,9 @@ class NewDirectoryNode:
     def get_verifier(self):
         return self._uri.get_verifier().to_string()
 
-    def check(self):
+    def check(self, verify=False, repair=False):
         """Perform a file check. See IChecker.check for details."""
-        return defer.succeed(None) # TODO
+        return self._node.check(verify, repair)
 
     def list(self):
         """I return a Deferred that fires with a dictionary mapping child
@@ -533,6 +534,45 @@ class NewDirectoryNode:
         d.addCallback(_got_list)
         return d
 
+    def deep_check(self, verify=False, repair=False):
+        # shallow-check each object first, then traverse children
+        root_si = self._node.get_storage_index()
+        results = DeepCheckResults(root_si)
+        found = set()
+        limiter = ConcurrencyLimiter(10)
+
+        d = self._add_deepcheck_from_node(self, results, found, limiter,
+                                          verify, repair)
+        d.addCallback(lambda res: results)
+        return d
+
+    def _add_deepcheck_from_node(self, node, results, found, limiter,
+                                 verify, repair):
+        verifier = node.get_verifier()
+        if verifier in found:
+            # avoid loops
+            return None
+        found.add(verifier)
+
+        d = limiter.add(node.check, verify, repair)
+        d.addCallback(results.add_check)
+
+        if IDirectoryNode.providedBy(node):
+            d.addCallback(lambda res: node.list())
+            def _got_children(children):
+                dl = []
+                for name, (child, metadata) in children.iteritems():
+                    d2 = self._add_deepcheck_from_node(child, results,
+                                                       found, limiter,
+                                                       verify, repair)
+                    if d2:
+                        dl.append(d2)
+                if dl:
+                    return defer.DeferredList(dl, fireOnOneErrback=True)
+            d.addCallback(_got_children)
+        return d
+
+
 class DeepStats:
     def __init__(self):
         self.stats = {}
