@@ -21,144 +21,10 @@ URIExtensionData = StringConstraint(1000)
 Number = IntegerConstraint(8) # 2**(8*8) == 16EiB ~= 18e18 ~= 18 exabytes
 Offset = Number
 ReadSize = int # the 'int' constraint is 2**31 == 2Gib -- large files are processed in not-so-large increments
+WriteEnablerSecret = Hash # used to protect mutable bucket modifications
 LeaseRenewSecret = Hash # used to protect bucket lease renewal requests
 LeaseCancelSecret = Hash # used to protect bucket lease cancellation requests
 
-# Announcements are (FURL, service_name, remoteinterface_name,
-#                    nickname, my_version, oldest_supported)
-#  the (FURL, service_name, remoteinterface_name) refer to the service being
-#  announced. The (nickname, my_version, oldest_supported) refer to the
-#  client as a whole. The my_version/oldest_supported strings can be parsed
-#  by an allmydata.util.version.Version instance, and then compared. The
-#  first goal is to make sure that nodes are not confused by speaking to an
-#  incompatible peer. The second goal is to enable the development of
-#  backwards-compatibility code.
-
-Announcement = TupleOf(FURL, str, str,
-                       str, str, str)
-
-class RIIntroducerSubscriberClient(RemoteInterface):
-    __remote_name__ = "RIIntroducerSubscriberClient.tahoe.allmydata.com"
-
-    def announce(announcements=SetOf(Announcement)):
-        """I accept announcements from the publisher."""
-        return None
-
-    def set_encoding_parameters(parameters=(int, int, int)):
-        """Advise the client of the recommended k-of-n encoding parameters
-        for this grid. 'parameters' is a tuple of (k, desired, n), where 'n'
-        is the total number of shares that will be created for any given
-        file, while 'k' is the number of shares that must be retrieved to
-        recover that file, and 'desired' is the minimum number of shares that
-        must be placed before the uploader will consider its job a success.
-        n/k is the expansion ratio, while k determines the robustness.
-
-        Introducers should specify 'n' according to the expected size of the
-        grid (there is no point to producing more shares than there are
-        peers), and k according to the desired reliability-vs-overhead goals.
-
-        Note that setting k=1 is equivalent to simple replication.
-        """
-        return None
-
-# When Foolscap can handle multiple interfaces (Foolscap#17), the
-# full-powered introducer will implement both RIIntroducerPublisher and
-# RIIntroducerSubscriberService. Until then, we define
-# RIIntroducerPublisherAndSubscriberService as a combination of the two, and
-# make everybody use that.
-
-class RIIntroducerPublisher(RemoteInterface):
-    """To publish a service to the world, connect to me and give me your
-    announcement message. I will deliver a copy to all connected subscribers."""
-    __remote_name__ = "RIIntroducerPublisher.tahoe.allmydata.com"
-
-    def publish(announcement=Announcement):
-        # canary?
-        return None
-
-class RIIntroducerSubscriberService(RemoteInterface):
-    __remote_name__ = "RIIntroducerSubscriberService.tahoe.allmydata.com"
-
-    def subscribe(subscriber=RIIntroducerSubscriberClient, service_name=str):
-        """Give me a subscriber reference, and I will call its new_peers()
-        method will any announcements that match the desired service name. I
-        will ignore duplicate subscriptions.
-        """
-        return None
-
-class RIIntroducerPublisherAndSubscriberService(RemoteInterface):
-    __remote_name__ = "RIIntroducerPublisherAndSubscriberService.tahoe.allmydata.com"
-    def publish(announcement=Announcement):
-        return None
-    def subscribe(subscriber=RIIntroducerSubscriberClient, service_name=str):
-        return None
-
-class IIntroducerClient(Interface):
-    """I provide service introduction facilities for a node. I help nodes
-    publish their services to the rest of the world, and I help them learn
-    about services available on other nodes."""
-
-    def publish(furl, service_name, remoteinterface_name):
-        """Once you call this, I will tell the world that the Referenceable
-        available at FURL is available to provide a service named
-        SERVICE_NAME. The precise definition of the service being provided is
-        identified by the Foolscap 'remote interface name' in the last
-        parameter: this is supposed to be a globally-unique string that
-        identifies the RemoteInterface that is implemented."""
-
-    def subscribe_to(service_name):
-        """Call this if you will eventually want to use services with the
-        given SERVICE_NAME. This will prompt me to subscribe to announcements
-        of those services. You can pick up the announcements later by calling
-        get_all_connections_for() or get_permuted_peers().
-        """
-
-    def get_all_connections():
-        """Return a frozenset of (nodeid, service_name, rref) tuples, one for
-        each active connection we've established to a remote service. This is
-        mostly useful for unit tests that need to wait until a certain number
-        of connections have been made."""
-
-    def get_all_connectors():
-        """Return a dict that maps from (nodeid, service_name) to a
-        RemoteServiceConnector instance for all services that we are actively
-        trying to connect to. Each RemoteServiceConnector has the following
-        public attributes::
-
-          service_name: the type of service provided, like 'storage'
-          announcement_time: when we first heard about this service
-          last_connect_time: when we last established a connection
-          last_loss_time: when we last lost a connection
-
-          version: the peer's version, from the most recent connection
-          oldest_supported: the peer's oldest supported version, same
-
-          rref: the RemoteReference, if connected, otherwise None
-          remote_host: the IAddress, if connected, otherwise None
-
-        This method is intended for monitoring interfaces, such as a web page
-        which describes connecting and connected peers.
-        """
-
-    def get_all_peerids():
-        """Return a frozenset of all peerids to whom we have a connection (to
-        one or more services) established. Mostly useful for unit tests."""
-
-    def get_all_connections_for(service_name):
-        """Return a frozenset of (nodeid, service_name, rref) tuples, one
-        for each active connection that provides the given SERVICE_NAME."""
-
-    def get_permuted_peers(service_name, key):
-        """Returns an ordered list of (peerid, rref) tuples, selecting from
-        the connections that provide SERVICE_NAME, using a hash-based
-        permutation keyed by KEY. This randomizes the service list in a
-        repeatable way, to distribute load over many peers.
-        """
-
-    def connected_to_introducer():
-        """Returns a boolean, True if we are currently connected to the
-        introducer, False if not."""
-
 class RIStubClient(RemoteInterface):
     """Each client publishes a service announcement for a dummy object called
     the StubClient. This object doesn't actually offer any services, but the
@@ -245,11 +111,21 @@ class RIStorageServer(RemoteInterface):
         return TupleOf(SetOf(int, maxLength=MAX_BUCKETS),
                        DictOf(int, RIBucketWriter, maxKeys=MAX_BUCKETS))
 
+    def add_lease(storage_index=StorageIndex,
+                  renew_secret=LeaseRenewSecret,
+                  cancel_secret=LeaseCancelSecret):
+        """
+        Add a new lease on the given bucket. If the renew_secret matches an
+        existing lease, that lease will be renewed instead.
+        """
+        return None
+
     def renew_lease(storage_index=StorageIndex, renew_secret=LeaseRenewSecret):
         """
         Renew the lease on a given bucket. Some networks will use this, some
         will not.
         """
+        return None
 
     def cancel_lease(storage_index=StorageIndex,
                      cancel_secret=LeaseCancelSecret):
@@ -257,6 +133,7 @@ class RIStorageServer(RemoteInterface):
         Cancel the lease on a given bucket. If this was the last lease on the
         bucket, the bucket will be deleted.
         """
+        return None
 
     def get_buckets(storage_index=StorageIndex):
         return DictOf(int, RIBucketReader, maxKeys=MAX_BUCKETS)
@@ -271,7 +148,9 @@ class RIStorageServer(RemoteInterface):
         return DictOf(int, ReadData) # shnum -> results
 
     def slot_testv_and_readv_and_writev(storage_index=StorageIndex,
-                                        secrets=TupleOf(Hash, Hash, Hash),
+                                        secrets=TupleOf(WriteEnablerSecret,
+                                                        LeaseRenewSecret,
+                                                        LeaseCancelSecret),
                                         tw_vectors=TestAndWriteVectorsForShares,
                                         r_vector=ReadVector,
                                         ):
@@ -335,8 +214,9 @@ class RIStorageServer(RemoteInterface):
         for each element of the read vector.
 
         If the write_enabler is wrong, this will raise BadWriteEnablerError.
-        To enable share migration, the exception will have the nodeid used
-        for the old write enabler embedded in it, in the following string::
+        To enable share migration (using update_write_enabler), the exception
+        will have the nodeid used for the old write enabler embedded in it,
+        in the following string::
 
          The write enabler was recorded by nodeid '%s'.
 
@@ -531,8 +411,22 @@ class IFilesystemNode(Interface):
         it holds a share for the file or directory.
         """
 
-    def check():
-        """Perform a file check. See IChecker.check for details."""
+    def check(verify=False, repair=False):
+        """Perform a file check. See IChecker.check for details.
+
+        The default mode is named 'check' and simply asks each server whether
+        or not it has a share, and believes the answers. If verify=True, this
+        switches into the 'verify' mode, in which every byte of every share
+        is retrieved, and all hashes are verified. This uses much more
+        network and disk bandwidth than simple checking, especially for large
+        files, but will catch disk errors.
+
+        The default repair=False argument means that files which are not
+        perfectly healthy will be reported, but not fixed. When repair=True,
+        the node will attempt to repair the file first. The results will
+        indicate the initial status of the file in either case. If repair was
+        attempted, the results will indicate that too.
+        """
 
     def is_readonly():
         """Return True if this reference provides mutable access to the given
@@ -1263,6 +1157,9 @@ class IDecoder(Interface):
         """
 
 class IDownloadTarget(Interface):
+    # Note that if the IDownloadTarget is also an IConsumable, the downloader
+    # will register itself as a producer. This allows the target to invoke
+    # downloader.pauseProducing, resumeProducing, and stopProducing.
     def open(size):
         """Called before any calls to write() or close(). If an error
         occurs before any data is available, fail() may be called without
@@ -1517,67 +1414,130 @@ class IUploader(Interface):
     def upload_ssk(write_capability, new_version, uploadable):
         """TODO: how should this work?"""
 
-class IChecker(Interface):
-    def check(uri_to_check):
-        """Accepts an IVerifierURI, and checks upon the health of its target.
+class ICheckable(Interface):
+    def check(verify=False, repair=False):
+        """Check upon my health, optionally repairing any problems.
 
-        For now, uri_to_check must be an IVerifierURI. In the future we
-        expect to relax that to be anything that can be adapted to
-        IVerifierURI (like read-only or read-write dirnode/filenode URIs).
+        This returns a Deferred that fires with an instance that provides
+        ICheckerResults.
 
-        This returns a Deferred. For dirnodes, this fires with either True or
-        False (dirnodes are not distributed, so their health is a boolean).
+        Filenodes and dirnodes (which provide IFilesystemNode) are also
+        checkable. Instances that represent verifier-caps will be checkable
+        but not downloadable. Some objects (like LIT files) do not actually
+        live in the grid, and their checkers indicate a healthy result.
 
-        For filenodes, this fires with a tuple of (needed_shares,
-        total_shares, found_shares, sharemap). The first three are ints. The
-        basic health of the file is found_shares / needed_shares: if less
-        than 1.0, the file is unrecoverable.
+        If verify=False, a relatively lightweight check will be performed: I
+        will ask all servers if they have a share for me, and I will believe
+        whatever they say. If there are at least N distinct shares on the
+        grid, my results will indicate r.is_healthy()==True. This requires a
+        roundtrip to each server, but does not transfer very much data, so
+        the network bandwidth is fairly low.
 
-        The sharemap has a key for each sharenum. The value is a list of
-        (binary) nodeids who hold that share. If two shares are kept on the
-        same nodeid, they will fail as a pair, and overall reliability is
-        decreased.
+        If verify=True, a more resource-intensive check will be performed:
+        every share will be downloaded, and the hashes will be validated on
+        every bit. I will ignore any shares that failed their hash checks. If
+        there are at least N distinct valid shares on the grid, my results
+        will indicate r.is_healthy()==True. This requires N/k times as much
+        download bandwidth (and server disk IO) as a regular download. If a
+        storage server is holding a corrupt share, or is experiencing memory
+        failures during retrieval, or is malicious or buggy, then
+        verification will detect the problem, but checking will not.
 
-        The IChecker instance remembers the results of the check. By default,
-        these results are stashed in RAM (and are forgotten at shutdown). If
-        a file named 'checker_results.db' exists in the node's basedir, it is
-        used as a sqlite database of results, making them persistent across
-        runs. To start using this feature, just 'touch checker_results.db',
-        and the node will initialize it properly the next time it is started.
+        If repair=True, then a non-healthy result will cause an immediate
+        repair operation, to generate and upload new shares. After repair,
+        the file will be as healthy as we can make it. Details about what
+        sort of repair is done will be put in the checker results. My
+        Deferred will not fire until the repair is complete.
+
+        TODO: any problems seen during checking will be reported to the
+        health-manager.furl, a centralized object which is responsible for
+        figuring out why files are unhealthy so corrective action can be
+        taken.
         """
 
-    def verify(uri_to_check):
-        """Accepts an IVerifierURI, and verifies the crypttext of the target.
+    def deep_check(verify=False, repair=False):
+        """Check upon the health of me and everything I can reach.
 
-        This is a more-intensive form of checking. For verification, the
-        file's crypttext contents are retrieved, and the associated hash
-        checks are performed. If a storage server is holding a corrupted
-        share, verification will detect the problem, but checking will not.
-        This returns a Deferred that fires with True if the crypttext hashes
-        look good, and will probably raise an exception if anything goes
-        wrong.
+        This is a recursive form of check(), useable on dirnodes. (it can be
+        called safely on filenodes too, but only checks the one object).
 
-        For dirnodes, 'verify' is the same as 'check', so the Deferred will
-        fire with True or False.
-
-        Verification currently only uses a minimal subset of peers, so a lot
-        of share corruption will not be caught by it. We expect to improve
-        this in the future.
+        I return a Deferred that fires with an IDeepCheckResults object.
         """
 
-    def checker_results_for(uri_to_check):
-        """Accepts an IVerifierURI, and returns a list of previously recorded
-        checker results. This method performs no checking itself: it merely
-        reports the results of checks that have taken place in the past.
+class ICheckerResults(Interface):
+    """I contain the detailed results of a check/verify/repair operation.
 
-        Each element of the list is a two-entry tuple: (when, results).
-        The 'when' values are timestamps (float seconds since epoch), and the
-        results are as defined in the check() method.
+    The IFilesystemNode.check()/verify()/repair() methods all return
+    instances that provide ICheckerResults.
+    """
 
-        Note: at the moment, this is specified to return synchronously. We
-        might need to back away from this in the future.
+    def is_healthy():
+        """Return a bool, True if the file is fully healthy, False if it is
+        damaged in any way."""
+
+    def get_storage_index_string():
+        """Return a string with the abbreviated storage index."""
+    def get_mutability_string():
+        """Return a string with 'mutable' or 'immutable'."""
+
+    def to_string():
+        """Return a string that describes the detailed results of the
+        check/verify operation. This string will be displayed on a page all
+        by itself."""
+
+    # The old checker results (for only immutable files) were described
+    # with this:
+    #    For filenodes, this fires with a tuple of (needed_shares,
+    #    total_shares, found_shares, sharemap). The first three are ints. The
+    #    basic health of the file is found_shares / needed_shares: if less
+    #    than 1.0, the file is unrecoverable.
+    #
+    #    The sharemap has a key for each sharenum. The value is a list of
+    #    (binary) nodeids who hold that share. If two shares are kept on the
+    #    same nodeid, they will fail as a pair, and overall reliability is
+    #    decreased.
+
+class IDeepCheckResults(Interface):
+    """I contain the results of a deep-check operation.
+
+    This is returned by a call to ICheckable.deep_check().
+    """
+
+    def get_root_storage_index_string():
+        """Return the storage index (abbreviated human-readable string) of
+        the first object checked."""
+    def count_objects_checked():
+        """Return the number of objects that were checked."""
+    def count_objects_healthy():
+        """Return the number of objects that were fully healthy."""
+    def count_repairs_attempted():
+        """Return the number of repair operations that were attempted."""
+    def count_repairs_successful():
+        """Return the number of repair operations that succeeded in bringing
+        the object back up to full health."""
+    def get_server_problems():
+        """Return a dict, mapping server nodeid to a count of how many
+        problems involved that server."""
+    def get_problems():
+        """Return a list of ICheckerResults, one for each object that
+        was not fully healthy."""
+
+class IRepairable(Interface):
+    def repair(checker_results):
+        """Attempt to repair the given object. Returns a Deferred that fires
+        with a IRepairResults object.
+
+        I must be called with an object that implements ICheckerResults, as
+        proof that you have actually discovered a problem with this file. I
+        will use the data in the checker results to guide the repair process,
+        such as which servers provided bad data and should therefore be
+        avoided.
         """
 
+class IRepairResults(Interface):
+    """I contain the results of a repair operation."""
+
+
 class IClient(Interface):
     def upload(uploadable):
         """Upload some data into a CHK, get back the UploadResults for it.
