@@ -6,11 +6,13 @@ from zope.interface import implements
 from twisted.internet import defer, reactor
 from twisted.python import log
 from foolscap.eventual import eventually
-from allmydata.interfaces import IMutableFileNode, IMutableFileURI
+from allmydata.interfaces import IMutableFileNode, IMutableFileURI, \
+     ICheckable, ICheckerResults
 from allmydata.util import hashutil
 from allmydata.util.assertutil import precondition
 from allmydata.uri import WriteableSSKFileURI
-from allmydata.encode import NotEnoughSharesError
+from allmydata.immutable.encode import NotEnoughSharesError
+from allmydata.immutable.checker import DeepCheckResults
 from pycryptopp.publickey import rsa
 from pycryptopp.cipher.aes import AES
 
@@ -19,6 +21,8 @@ from common import MODE_READ, MODE_WRITE, UnrecoverableFileError, \
      ResponseCache, UncoordinatedWriteError
 from servermap import ServerMap, ServermapUpdater
 from retrieve import Retrieve
+from checker import MutableChecker
+from repair import Repairer
 
 
 class BackoffAgent:
@@ -46,9 +50,10 @@ class BackoffAgent:
 # use client.create_mutable_file() to make one of these
 
 class MutableFileNode:
-    implements(IMutableFileNode)
+    implements(IMutableFileNode, ICheckable)
     SIGNATURE_KEY_SIZE = 2048
     DEFAULT_ENCODING = (3, 10)
+    checker_class = MutableChecker
 
     def __init__(self, client):
         self._client = client
@@ -183,6 +188,8 @@ class MutableFileNode:
     def get_total_shares(self):
         return self._total_shares
 
+    ####################################
+    # IFilesystemNode
 
     def get_uri(self):
         return self._uri.to_string()
@@ -234,10 +241,33 @@ class MutableFileNode:
         return d
 
     #################################
+    # ICheckable
 
-    def check(self):
-        verifier = self.get_verifier()
-        return self._client.getServiceNamed("checker").check(verifier)
+    def check(self, verify=False, repair=False):
+        checker = self.checker_class(self)
+        return checker.check(verify, repair)
+
+    def deep_check(self, verify=False, repair=False):
+        d = self.check(verify, repair)
+        def _done(r):
+            dr = DeepCheckResults(self.get_storage_index())
+            dr.add_check(r)
+            return dr
+        d.addCallback(_done)
+        return d
+
+    #################################
+    # IRepairable
+
+    def repair(self, checker_results):
+        assert ICheckerResults(checker_results)
+        r = Repairer(self, checker_results)
+        d = r.start()
+        return d
+
+
+    #################################
+    # IMutableFileNode
 
     # allow the use of IDownloadTarget
     def download(self, target):
