@@ -12,6 +12,7 @@ class DumpOptions(usage.Options):
 
 def dump_share(config, out=sys.stdout, err=sys.stderr):
     from allmydata import uri, storage
+    from allmydata.util import base32
 
     # check the version, to see if we have a mutable or immutable share
     print >>out, "share filename: %s" % config['filename']
@@ -61,6 +62,18 @@ def dump_share(config, out=sys.stdout, err=sys.stderr):
         for k in sorted(leftover):
             print >>out, "%20s: %s" % (k, unpacked[k])
 
+    # the storage index isn't stored in the share itself, so we depend upon
+    # knowing the parent directory name to get it
+    pieces = config['filename'].split(os.sep)
+    if len(pieces) >= 2 and base32.could_be_base32_encoded(pieces[-2]):
+        storage_index = base32.a2b(pieces[-2])
+        uri_extension_hash = base32.a2b(unpacked["UEB_hash"])
+        u = uri.CHKFileVerifierURI(storage_index, uri_extension_hash,
+                                   unpacked["needed_shares"],
+                                   unpacked["total_shares"], unpacked["size"])
+        verify_cap = u.to_string()
+        print >>out, "%20s: %s" % ("verify-cap", verify_cap)
+
     sizes = {}
     sizes['data'] = bp._data_size
     sizes['validation'] = (offsets['uri_extension'] -
@@ -75,10 +88,9 @@ def dump_share(config, out=sys.stdout, err=sys.stderr):
     leases = list(f.iter_leases())
     if leases:
         for i,lease in enumerate(leases):
-            (owner_num, renew_secret, cancel_secret, expiration_time) = lease
-            when = format_expiration_time(expiration_time)
-            print >>out, " Lease #%d: owner=%d, expire in %s" % (i, owner_num,
-                                                                 when)
+            when = format_expiration_time(lease.expiration_time)
+            print >>out, " Lease #%d: owner=%d, expire in %s" \
+                  % (i, lease.owner_num, when)
     else:
         print >>out, " No leases."
 
@@ -124,15 +136,15 @@ def dump_mutable_share(config, out, err):
     print >>out, " container_size: %d" % container_size
     print >>out, " data_length: %d" % data_length
     if leases:
-        for (leasenum, (oid,et,rs,cs,anid)) in leases:
+        for (leasenum, lease) in leases:
             print >>out
             print >>out, " Lease #%d:" % leasenum
-            print >>out, "  ownerid: %d" % oid
-            when = format_expiration_time(et)
+            print >>out, "  ownerid: %d" % lease.owner_num
+            when = format_expiration_time(lease.expiration_time)
             print >>out, "  expires in %s" % when
-            print >>out, "  renew_secret: %s" % base32.b2a(rs)
-            print >>out, "  cancel_secret: %s" % base32.b2a(cs)
-            print >>out, "  secrets are for nodeid: %s" % idlib.nodeid_b2a(anid)
+            print >>out, "  renew_secret: %s" % base32.b2a(lease.renew_secret)
+            print >>out, "  cancel_secret: %s" % base32.b2a(lease.cancel_secret)
+            print >>out, "  secrets are for nodeid: %s" % idlib.nodeid_b2a(lease.nodeid)
     else:
         print >>out, "No leases."
     print >>out
@@ -145,7 +157,8 @@ def dump_mutable_share(config, out, err):
 def dump_SDMF_share(offset, length, config, out, err):
     from allmydata.mutable.layout import unpack_share
     from allmydata.mutable.common import NeedMoreDataError
-    from allmydata.util import base32
+    from allmydata.util import base32, hashutil
+    from allmydata.uri import SSKVerifierURI
 
     f = open(config['filename'], "rb")
     f.seek(offset)
@@ -183,6 +196,16 @@ def dump_SDMF_share(offset, length, config, out, err):
     print >>out, "  share_hash_chain: %s" % share_hash_ids
     print >>out, "  block_hash_tree: %d nodes" % len(block_hash_tree)
 
+    # the storage index isn't stored in the share itself, so we depend upon
+    # knowing the parent directory name to get it
+    pieces = config['filename'].split(os.sep)
+    if len(pieces) >= 2 and base32.could_be_base32_encoded(pieces[-2]):
+        storage_index = base32.a2b(pieces[-2])
+        fingerprint = hashutil.ssk_pubkey_fingerprint_hash(pubkey)
+        u = SSKVerifierURI(storage_index, fingerprint)
+        verify_cap = u.to_string()
+        print >>out, "  verify-cap:", verify_cap
+
     print >>out
 
 
@@ -378,10 +401,8 @@ def describe_share(abs_sharefile, si_s, shnum_s, now, out, err):
         extra_lease_offset = m._read_extra_lease_offset(f)
         container_size = extra_lease_offset - m.DATA_OFFSET
         leases = list(m._enumerate_leases(f))
-        expiration_time = min( [expiration_time
-                                for (leasenum,
-                                     (ownerid, expiration_time, rs, cs, nodeid))
-                                in leases] )
+        expiration_time = min( [lease[1].expiration_time
+                                for lease in leases] )
         expiration = max(0, expiration_time - now)
 
         share_type = "unknown"
@@ -424,9 +445,8 @@ def describe_share(abs_sharefile, si_s, shnum_s, now, out, err):
         length = struct.unpack(">L", sf.read_share_data(seek, 4))[0]
         seek += 4
         UEB_data = sf.read_share_data(seek, length)
-        expiration_time = min( [expiration_time
-                                for (ownerid, rs, cs, expiration_time)
-                                in sf.iter_leases()] )
+        expiration_time = min( [lease.expiration_time
+                                for lease in sf.iter_leases()] )
         expiration = max(0, expiration_time - now)
 
         unpacked = uri.unpack_extension_readable(UEB_data)
@@ -449,8 +469,15 @@ def catalog_shares(config, out=sys.stdout, err=sys.stderr):
     now = time.time()
     for d in config.nodedirs:
         d = os.path.join(os.path.expanduser(d), "storage/shares")
-        if os.path.exists(d):
-            for abbrevdir in os.listdir(d):
+        try:
+            abbrevs = os.listdir(d)
+        except EnvironmentError:
+            # ignore nodes that have storage turned off altogether
+            pass
+        else:
+            for abbrevdir in abbrevs:
+                if abbrevdir == "incoming":
+                    continue
                 abbrevdir = os.path.join(d, abbrevdir)
                 for si_s in os.listdir(abbrevdir):
                     si_dir = os.path.join(abbrevdir, si_s)
