@@ -56,6 +56,45 @@ def storage_index_to_dir(storageindex):
     sia = si_b2a(storageindex)
     return os.path.join(sia[:2], sia)
 
+class LeaseInfo:
+    def __init__(self, owner_num=None, renew_secret=None, cancel_secret=None,
+                 expiration_time=None, nodeid=None):
+        self.owner_num = owner_num
+        self.renew_secret = renew_secret
+        self.cancel_secret = cancel_secret
+        self.expiration_time = expiration_time
+        if nodeid is not None:
+            assert isinstance(nodeid, str)
+            assert len(nodeid) == 20
+        self.nodeid = nodeid
+
+    def from_immutable_data(self, data):
+        (self.owner_num,
+         self.renew_secret,
+         self.cancel_secret,
+         self.expiration_time) = struct.unpack(">L32s32sL", data)
+        self.nodeid = None
+        return self
+    def to_immutable_data(self):
+        return struct.pack(">L32s32sL",
+                           self.owner_num,
+                           self.renew_secret, self.cancel_secret,
+                           int(self.expiration_time))
+
+    def to_mutable_data(self):
+        return struct.pack(">LL32s32s20s",
+                           self.owner_num,
+                           int(self.expiration_time),
+                           self.renew_secret, self.cancel_secret,
+                           self.nodeid)
+    def from_mutable_data(self, data):
+        (self.owner_num,
+         self.expiration_time,
+         self.renew_secret, self.cancel_secret,
+         self.nodeid) = struct.unpack(">LL32s32s20s", data)
+        return self
+
+
 class ShareFile:
     LEASE_SIZE = struct.calcsize(">L32s32sL")
 
@@ -69,6 +108,9 @@ class ShareFile:
         self._data_offset = 0xc
         self._lease_offset = 0xc + self._size
 
+    def unlink(self):
+        os.unlink(self.home)
+
     def read_share_data(self, offset, length):
         precondition(offset >= 0)
         precondition(offset+length <= self._size)
@@ -88,13 +130,10 @@ class ShareFile:
         f.close()
 
     def _write_lease_record(self, f, lease_number, lease_info):
-        (owner_num, renew_secret, cancel_secret, expiration_time) = lease_info
         offset = self._lease_offset + lease_number * self.LEASE_SIZE
         f.seek(offset)
         assert f.tell() == offset
-        f.write(struct.pack(">L32s32sL",
-                            owner_num, renew_secret, cancel_secret,
-                            int(expiration_time)))
+        f.write(lease_info.to_immutable_data())
 
     def _read_num_leases(self, f):
         f.seek(0x08)
@@ -117,7 +156,7 @@ class ShareFile:
         for i in range(num_leases):
             data = f.read(self.LEASE_SIZE)
             if data:
-                yield struct.unpack(">L32s32sL", data)
+                yield LeaseInfo().from_immutable_data(data)
 
     def add_lease(self, lease_info):
         f = open(self.home, 'rb+')
@@ -127,36 +166,39 @@ class ShareFile:
         f.close()
 
     def renew_lease(self, renew_secret, new_expire_time):
-        for i,(on,rs,cs,et) in enumerate(self.iter_leases()):
-            if rs == renew_secret:
+        for i,lease in enumerate(self.iter_leases()):
+            if lease.renew_secret == renew_secret:
                 # yup. See if we need to update the owner time.
-                if new_expire_time > et:
+                if new_expire_time > lease.expiration_time:
                     # yes
-                    new_lease = (on,rs,cs,new_expire_time)
+                    lease.expiration_time = new_expire_time
                     f = open(self.home, 'rb+')
-                    self._write_lease_record(f, i, new_lease)
+                    self._write_lease_record(f, i, lease)
                     f.close()
                 return
         raise IndexError("unable to renew non-existent lease")
 
     def add_or_renew_lease(self, lease_info):
-        owner_num, renew_secret, cancel_secret, expire_time = lease_info
         try:
-            self.renew_lease(renew_secret, expire_time)
+            self.renew_lease(lease_info.renew_secret,
+                             lease_info.expiration_time)
         except IndexError:
             self.add_lease(lease_info)
 
+
     def cancel_lease(self, cancel_secret):
-        """Remove a lease with the given cancel_secret. Return
-        (num_remaining_leases, space_freed). Raise IndexError if there was no
-        lease with the given cancel_secret."""
+        """Remove a lease with the given cancel_secret. If the last lease is
+        cancelled, the file will be removed. Return the number of bytes that
+        were freed (by truncating the list of leases, and possibly by
+        deleting the file. Raise IndexError if there was no lease with the
+        given cancel_secret.
+        """
 
         leases = list(self.iter_leases())
         num_leases = len(leases)
         num_leases_removed = 0
-        for i,lease_info in enumerate(leases[:]):
-            (on,rs,cs,et) = lease_info
-            if cs == cancel_secret:
+        for i,lease in enumerate(leases[:]):
+            if lease.cancel_secret == cancel_secret:
                 leases[i] = None
                 num_leases_removed += 1
         if not num_leases_removed:
@@ -172,7 +214,11 @@ class ShareFile:
             self._write_num_leases(f, len(leases))
             self._truncate_leases(f, len(leases))
             f.close()
-        return len(leases), self.LEASE_SIZE * num_leases_removed
+        space_freed = self.LEASE_SIZE * num_leases_removed
+        if not len(leases):
+            space_freed += os.stat(self.home)[stat.ST_SIZE]
+            self.unlink()
+        return space_freed
 
 
 class BucketWriter(Referenceable):
@@ -203,21 +249,43 @@ class BucketWriter(Referenceable):
         return self._size
 
     def remote_write(self, offset, data):
+        start = time.time()
         precondition(not self.closed)
         if self.throw_out_all_data:
             return
         self._sharefile.write_share_data(offset, data)
+        self.ss.add_latency("write", time.time() - start)
+        self.ss.count("write")
 
     def remote_close(self):
         precondition(not self.closed)
+        start = time.time()
 
         fileutil.make_dirs(os.path.dirname(self.finalhome))
         fileutil.rename(self.incominghome, self.finalhome)
         try:
+            # self.incominghome is like storage/shares/incoming/ab/abcde/4 .
+            # We try to delete the parent (.../ab/abcde) to avoid leaving
+            # these directories lying around forever, but the delete might
+            # fail if we're working on another share for the same storage
+            # index (like ab/abcde/5). The alternative approach would be to
+            # use a hierarchy of objects (PrefixHolder, BucketHolder,
+            # ShareWriter), each of which is responsible for a single
+            # directory on disk, and have them use reference counting of
+            # their children to know when they should do the rmdir. This
+            # approach is simpler, but relies on os.rmdir refusing to delete
+            # a non-empty directory. Do *not* use fileutil.rm_dir() here!
             os.rmdir(os.path.dirname(self.incominghome))
+            # we also delete the grandparent (prefix) directory, .../ab ,
+            # again to avoid leaving directories lying around. This might
+            # fail if there is another bucket open that shares a prefix (like
+            # ab/abfff).
             os.rmdir(os.path.dirname(os.path.dirname(self.incominghome)))
-            os.rmdir(os.path.dirname(os.path.dirname(os.path.dirname(self.incominghome))))
+            # we leave the great-grandparent (incoming/) directory in place.
         except EnvironmentError:
+            # ignore the "can't rmdir because the directory is not empty"
+            # exceptions, those are normal consequences of the
+            # above-mentioned conditions.
             pass
         self._sharefile = None
         self.closed = True
@@ -225,6 +293,8 @@ class BucketWriter(Referenceable):
 
         filelen = os.stat(self.finalhome)[stat.ST_SIZE]
         self.ss.bucket_writer_closed(self, filelen)
+        self.ss.add_latency("close", time.time() - start)
+        self.ss.count("close")
 
     def _disconnected(self):
         if not self.closed:
@@ -236,6 +306,7 @@ class BucketWriter(Referenceable):
         if not self.closed:
             self._canary.dontNotifyOnDisconnect(self._disconnect_marker)
         self._abort()
+        self.ss.count("abort")
 
     def _abort(self):
         if self.closed:
@@ -252,11 +323,16 @@ class BucketWriter(Referenceable):
 class BucketReader(Referenceable):
     implements(RIBucketReader)
 
-    def __init__(self, sharefname):
+    def __init__(self, ss, sharefname):
+        self.ss = ss
         self._share_file = ShareFile(sharefname)
 
     def remote_read(self, offset, length):
-        return self._share_file.read_share_data(offset, length)
+        start = time.time()
+        data = self._share_file.read_share_data(offset, length)
+        self.ss.add_latency("read", time.time() - start)
+        self.ss.count("read")
+        return data
 
 
 # the MutableShareFile is like the ShareFile, but used for mutable data. It
@@ -335,6 +411,9 @@ class MutableShareFile:
         # extra leases go here, none at creation
         f.close()
 
+    def unlink(self):
+        os.unlink(self.home)
+
     def _read_data_length(self, f):
         f.seek(self.DATA_LENGTH_OFFSET)
         (data_length,) = struct.unpack(">Q", f.read(8))
@@ -427,8 +506,6 @@ class MutableShareFile:
         return
 
     def _write_lease_record(self, f, lease_number, lease_info):
-        (ownerid, expiration_time,
-         renew_secret, cancel_secret, nodeid) = lease_info
         extra_lease_offset = self._read_extra_lease_offset(f)
         num_extra_leases = self._read_num_extra_leases(f)
         if lease_number < 4:
@@ -445,12 +522,10 @@ class MutableShareFile:
                       + (lease_number-4)*self.LEASE_SIZE)
         f.seek(offset)
         assert f.tell() == offset
-        f.write(struct.pack(">LL32s32s20s",
-                            ownerid, int(expiration_time),
-                            renew_secret, cancel_secret, nodeid))
+        f.write(lease_info.to_mutable_data())
 
     def _read_lease_record(self, f, lease_number):
-        # returns a 5-tuple of lease info, or None
+        # returns a LeaseInfo instance, or None
         extra_lease_offset = self._read_extra_lease_offset(f)
         num_extra_leases = self._read_num_extra_leases(f)
         if lease_number < 4:
@@ -464,10 +539,8 @@ class MutableShareFile:
         f.seek(offset)
         assert f.tell() == offset
         data = f.read(self.LEASE_SIZE)
-        lease_info = struct.unpack(">LL32s32s20s", data)
-        (ownerid, expiration_time,
-         renew_secret, cancel_secret, nodeid) = lease_info
-        if ownerid == 0:
+        lease_info = LeaseInfo().from_mutable_data(data)
+        if lease_info.owner_num == 0:
             return None
         return lease_info
 
@@ -516,16 +589,16 @@ class MutableShareFile:
     def renew_lease(self, renew_secret, new_expire_time):
         accepting_nodeids = set()
         f = open(self.home, 'rb+')
-        for (leasenum,(oid,et,rs,cs,anid)) in self._enumerate_leases(f):
-            if rs == renew_secret:
+        for (leasenum,lease) in self._enumerate_leases(f):
+            if lease.renew_secret == renew_secret:
                 # yup. See if we need to update the owner time.
-                if new_expire_time > et:
+                if new_expire_time > lease.expiration_time:
                     # yes
-                    new_lease = (oid,new_expire_time,rs,cs,anid)
-                    self._write_lease_record(f, leasenum, new_lease)
+                    lease.expiration_time = new_expire_time
+                    self._write_lease_record(f, leasenum, lease)
                 f.close()
                 return
-            accepting_nodeids.add(anid)
+            accepting_nodeids.add(lease.nodeid)
         f.close()
         # Return the accepting_nodeids set, to give the client a chance to
         # update the leases on a share which has been migrated from its
@@ -538,25 +611,31 @@ class MutableShareFile:
         raise IndexError(msg)
 
     def add_or_renew_lease(self, lease_info):
-        ownerid, expire_time, renew_secret, cancel_secret, anid = lease_info
         try:
-            self.renew_lease(renew_secret, expire_time)
+            self.renew_lease(lease_info.renew_secret,
+                             lease_info.expiration_time)
         except IndexError:
             self.add_lease(lease_info)
 
     def cancel_lease(self, cancel_secret):
-        """Remove any leases with the given cancel_secret. Return
-        (num_remaining_leases, space_freed). Raise IndexError if there was no
-        lease with the given cancel_secret."""
+        """Remove any leases with the given cancel_secret. If the last lease
+        is cancelled, the file will be removed. Return the number of bytes
+        that were freed (by truncating the list of leases, and possibly by
+        deleting the file. Raise IndexError if there was no lease with the
+        given cancel_secret."""
 
         accepting_nodeids = set()
         modified = 0
         remaining = 0
-        blank_lease = (0, 0, "\x00"*32, "\x00"*32, "\x00"*20)
+        blank_lease = LeaseInfo(owner_num=0,
+                                renew_secret="\x00"*32,
+                                cancel_secret="\x00"*32,
+                                expiration_time=0,
+                                nodeid="\x00"*20)
         f = open(self.home, 'rb+')
-        for (leasenum,(oid,et,rs,cs,anid)) in self._enumerate_leases(f):
-            accepting_nodeids.add(anid)
-            if cs == cancel_secret:
+        for (leasenum,lease) in self._enumerate_leases(f):
+            accepting_nodeids.add(lease.nodeid)
+            if lease.cancel_secret == cancel_secret:
                 self._write_lease_record(f, leasenum, blank_lease)
                 modified += 1
             else:
@@ -564,7 +643,11 @@ class MutableShareFile:
         if modified:
             freed_space = self._pack_leases(f)
             f.close()
-            return (remaining, freed_space)
+            if not remaining:
+                freed_space += os.stat(self.home)[stat.ST_SIZE]
+                self.unlink()
+            return freed_space
+
         msg = ("Unable to cancel non-existent lease. I have leases "
                "accepted by nodeids: ")
         msg += ",".join([("'%s'" % idlib.nodeid_b2a(anid))
@@ -704,6 +787,54 @@ class StorageServer(service.MultiService, Referenceable):
             log.msg(format="space measurement done, consumed=%(consumed)d bytes",
                     consumed=self.consumed,
                     parent=lp, facility="tahoe.storage")
+        self.latencies = {"allocate": [], # immutable
+                          "write": [],
+                          "close": [],
+                          "read": [],
+                          "renew": [],
+                          "cancel": [],
+                          "get": [],
+                          "writev": [], # mutable
+                          "readv": [],
+                          }
+
+    def count(self, name, delta=1):
+        if self.stats_provider:
+            self.stats_provider.count("storage_server." + name, delta)
+
+    def add_latency(self, category, latency):
+        a = self.latencies[category]
+        a.append(latency)
+        if len(a) > 1000:
+            self.latencies[category] = a[-1000:]
+
+    def get_latencies(self):
+        """Return a dict, indexed by category, that contains a dict of
+        latency numbers for each category. Each dict will contain the
+        following keys: mean, 01_0_percentile, 10_0_percentile,
+        50_0_percentile (median), 90_0_percentile, 95_0_percentile,
+        99_0_percentile, 99_9_percentile. If no samples have been collected
+        for the given category, then that category name will not be present
+        in the return value."""
+        # note that Amazon's Dynamo paper says they use 99.9% percentile.
+        output = {}
+        for category in self.latencies:
+            if not self.latencies[category]:
+                continue
+            stats = {}
+            samples = self.latencies[category][:]
+            samples.sort()
+            count = len(samples)
+            stats["mean"] = sum(samples) / count
+            stats["01_0_percentile"] = samples[int(0.01 * count)]
+            stats["10_0_percentile"] = samples[int(0.1 * count)]
+            stats["50_0_percentile"] = samples[int(0.5 * count)]
+            stats["90_0_percentile"] = samples[int(0.9 * count)]
+            stats["95_0_percentile"] = samples[int(0.95 * count)]
+            stats["99_0_percentile"] = samples[int(0.99 * count)]
+            stats["99_9_percentile"] = samples[int(0.999 * count)]
+            output[category] = stats
+        return output
 
     def log(self, *args, **kwargs):
         if "facility" not in kwargs:
@@ -729,6 +860,9 @@ class StorageServer(service.MultiService, Referenceable):
         stats = { 'storage_server.allocated': self.allocated_size(), }
         if self.consumed is not None:
             stats['storage_server.consumed'] = self.consumed
+        for category,ld in self.get_latencies().items():
+            for name,v in ld.items():
+                stats['storage_server.latencies.%s.%s' % (category, name)] = v
         return stats
 
     def allocated_size(self):
@@ -747,6 +881,8 @@ class StorageServer(service.MultiService, Referenceable):
         # owner_num is not for clients to set, but rather it should be
         # curried into the PersonalStorageServer instance that is dedicated
         # to a particular owner.
+        start = time.time()
+        self.count("allocate")
         alreadygot = set()
         bucketwriters = {} # k: shnum, v: BucketWriter
         si_dir = storage_index_to_dir(storage_index)
@@ -759,7 +895,9 @@ class StorageServer(service.MultiService, Referenceable):
         # separate database. Note that the lease should not be added until
         # the BucketWrite has been closed.
         expire_time = time.time() + 31*24*60*60
-        lease_info = (owner_num, renew_secret, cancel_secret, expire_time)
+        lease_info = LeaseInfo(owner_num,
+                               renew_secret, cancel_secret,
+                               expire_time, self.my_nodeid)
 
         space_per_bucket = allocated_size
         no_limits = self.sizelimit is None
@@ -778,6 +916,7 @@ class StorageServer(service.MultiService, Referenceable):
 
         if self.readonly_storage:
             # we won't accept new shares
+            self.add_latency("allocate", time.time() - start)
             return alreadygot, bucketwriters
 
         for shnum in sharenums:
@@ -809,13 +948,11 @@ class StorageServer(service.MultiService, Referenceable):
         if bucketwriters:
             fileutil.make_dirs(os.path.join(self.sharedir, si_dir))
 
+        self.add_latency("allocate", time.time() - start)
         return alreadygot, bucketwriters
 
-    def remote_renew_lease(self, storage_index, renew_secret):
-        new_expire_time = time.time() + 31*24*60*60
-        found_buckets = False
+    def _iter_share_files(self, storage_index):
         for shnum, filename in self._get_bucket_shares(storage_index):
-            found_buckets = True
             f = open(filename, 'rb')
             header = f.read(32)
             f.close()
@@ -827,54 +964,70 @@ class StorageServer(service.MultiService, Referenceable):
             elif header[:4] == struct.pack(">L", 1):
                 sf = ShareFile(filename)
             else:
-                pass # non-sharefile
+                continue # non-sharefile
+            yield sf
+
+    def remote_add_lease(self, storage_index, renew_secret, cancel_secret,
+                         owner_num=0):
+        start = time.time()
+        self.count("add-lease")
+        new_expire_time = time.time() + 31*24*60*60
+        lease_info = LeaseInfo(owner_num,
+                               renew_secret, cancel_secret,
+                               new_expire_time, self.my_nodeid)
+        found_buckets = False
+        for sf in self._iter_share_files(storage_index):
+            found_buckets = True
+            # note: if the share has been migrated, the renew_lease()
+            # call will throw an exception, with information to help the
+            # client update the lease.
+            sf.add_or_renew_lease(lease_info)
+        self.add_latency("add-lease", time.time() - start)
+        if not found_buckets:
+            raise IndexError("no such storage index to do add-lease")
+
+
+    def remote_renew_lease(self, storage_index, renew_secret):
+        start = time.time()
+        self.count("renew")
+        new_expire_time = time.time() + 31*24*60*60
+        found_buckets = False
+        for sf in self._iter_share_files(storage_index):
+            found_buckets = True
             sf.renew_lease(renew_secret, new_expire_time)
+        self.add_latency("renew", time.time() - start)
         if not found_buckets:
             raise IndexError("no such lease to renew")
 
     def remote_cancel_lease(self, storage_index, cancel_secret):
-        storagedir = os.path.join(self.sharedir, storage_index_to_dir(storage_index))
+        start = time.time()
+        self.count("cancel")
 
-        remaining_files = 0
         total_space_freed = 0
         found_buckets = False
-        for shnum, filename in self._get_bucket_shares(storage_index):
+        for sf in self._iter_share_files(storage_index):
             # note: if we can't find a lease on one share, we won't bother
             # looking in the others. Unless something broke internally
             # (perhaps we ran out of disk space while adding a lease), the
             # leases on all shares will be identical.
             found_buckets = True
-            f = open(filename, 'rb')
-            header = f.read(32)
-            f.close()
-            if header[:32] == MutableShareFile.MAGIC:
-                sf = MutableShareFile(filename, self)
-                # note: if the share has been migrated, the renew_lease()
-                # call will throw an exception, with information to help the
-                # client update the lease.
-            elif header[:4] == struct.pack(">L", 1):
-                sf = ShareFile(filename)
-            else:
-                pass # non-sharefile
-            # this raises IndexError if the lease wasn't present
-            remaining_leases, space_freed = sf.cancel_lease(cancel_secret)
-            total_space_freed += space_freed
-            if remaining_leases:
-                remaining_files += 1
-            else:
-                # now remove the sharefile. We'll almost certainly be
-                # removing the entire directory soon.
-                filelen = os.stat(filename)[stat.ST_SIZE]
-                os.unlink(filename)
-                total_space_freed += filelen
-        if not remaining_files:
-            fileutil.rm_dir(storagedir)
+            # this raises IndexError if the lease wasn't present XXXX
+            total_space_freed += sf.cancel_lease(cancel_secret)
+
+        if found_buckets:
+            storagedir = os.path.join(self.sharedir,
+                                      storage_index_to_dir(storage_index))
+            if not os.listdir(storagedir):
+                os.rmdir(storagedir)
+
         if self.consumed is not None:
             self.consumed -= total_space_freed
         if self.stats_provider:
-            self.stats_provider.count('storage_server.bytes_freed', total_space_freed)
+            self.stats_provider.count('storage_server.bytes_freed',
+                                      total_space_freed)
+        self.add_latency("cancel", time.time() - start)
         if not found_buckets:
-            raise IndexError("no such lease to cancel")
+            raise IndexError("no such storage index")
 
     def bucket_writer_closed(self, bw, consumed_size):
         if self.consumed is not None:
@@ -897,22 +1050,15 @@ class StorageServer(service.MultiService, Referenceable):
             # Commonly caused by there being no buckets at all.
             pass
 
-    def _get_incoming_shares(self, storage_index):
-        incomingdir = os.path.join(self.incomingdir, storage_index_to_dir(storage_index))
-        try:
-            for f in os.listdir(incomingdir):
-                if NUM_RE.match(f):
-                    filename = os.path.join(incomingdir, f)
-                    yield (int(f), filename)
-        except OSError:
-            pass
-
     def remote_get_buckets(self, storage_index):
+        start = time.time()
+        self.count("get")
         si_s = si_b2a(storage_index)
         log.msg("storage: get_buckets %s" % si_s)
         bucketreaders = {} # k: sharenum, v: BucketReader
         for shnum, filename in self._get_bucket_shares(storage_index):
-            bucketreaders[shnum] = BucketReader(filename)
+            bucketreaders[shnum] = BucketReader(self, filename)
+        self.add_latency("get", time.time() - start)
         return bucketreaders
 
     def get_leases(self, storage_index):
@@ -936,6 +1082,8 @@ class StorageServer(service.MultiService, Referenceable):
                                                secrets,
                                                test_and_write_vectors,
                                                read_vector):
+        start = time.time()
+        self.count("writev")
         si_s = si_b2a(storage_index)
         lp = log.msg("storage: slot_writev %s" % si_s)
         si_dir = storage_index_to_dir(storage_index)
@@ -994,14 +1142,14 @@ class StorageServer(service.MultiService, Referenceable):
             # and update the leases on all shares
             ownerid = 1 # TODO
             expire_time = time.time() + 31*24*60*60   # one month
-            my_nodeid = self.my_nodeid
-            anid = my_nodeid
-            lease_info = (ownerid, expire_time, renew_secret, cancel_secret,
-                          anid)
+            lease_info = LeaseInfo(ownerid,
+                                   renew_secret, cancel_secret,
+                                   expire_time, self.my_nodeid)
             for share in shares.values():
                 share.add_or_renew_lease(lease_info)
 
         # all done
+        self.add_latency("writev", time.time() - start)
         return (testv_is_good, read_data)
 
     def _allocate_slot_share(self, bucketdir, secrets, sharenum,
@@ -1015,6 +1163,8 @@ class StorageServer(service.MultiService, Referenceable):
         return share
 
     def remote_slot_readv(self, storage_index, shares, readv):
+        start = time.time()
+        self.count("readv")
         si_s = si_b2a(storage_index)
         lp = log.msg("storage: slot_readv %s %s" % (si_s, shares),
                      facility="tahoe.storage", level=log.OPERATIONAL)
@@ -1022,6 +1172,7 @@ class StorageServer(service.MultiService, Referenceable):
         # shares exist if there is a file for them
         bucketdir = os.path.join(self.sharedir, si_dir)
         if not os.path.isdir(bucketdir):
+            self.add_latency("readv", time.time() - start)
             return {}
         datavs = {}
         for sharenum_s in os.listdir(bucketdir):
@@ -1035,10 +1186,10 @@ class StorageServer(service.MultiService, Referenceable):
                 datavs[sharenum] = msf.readv(readv)
         log.msg("returning shares %s" % (datavs.keys(),),
                 facility="tahoe.storage", level=log.NOISY, parent=lp)
+        self.add_latency("readv", time.time() - start)
         return datavs
 
 
-
 # the code before here runs on the storage server, not the client
 # the code beyond here runs on the client, not the storage server
 
@@ -1243,6 +1394,10 @@ class ReadBucketProxy:
         # TODO: for small shares, read the whole bucket in start()
         d = self._read(0, 0x24)
         d.addCallback(self._parse_offsets)
+        def _started(res):
+            self._started = True
+            return res
+        d.addCallback(_started)
         return d
 
     def _parse_offsets(self, data):
