@@ -4,12 +4,13 @@ from cStringIO import StringIO
 from twisted.trial import unittest
 from twisted.internet import defer, reactor
 from twisted.python import failure
-from allmydata import uri, download, storage
+from allmydata import uri, storage
+from allmydata.immutable import download
+from allmydata.immutable.encode import NotEnoughSharesError
 from allmydata.util import base32, testutil, idlib
 from allmydata.util.idlib import shortnodeid_b2a
 from allmydata.util.hashutil import tagged_hash
 from allmydata.util.fileutil import make_dirs
-from allmydata.encode import NotEnoughSharesError
 from allmydata.interfaces import IURI, IMutableFileURI, IUploadable, \
      FileTooLargeError
 from foolscap.eventual import eventually, fireEventually
@@ -20,7 +21,7 @@ from allmydata.mutable.node import MutableFileNode, BackoffAgent
 from allmydata.mutable.common import DictOfSets, ResponseCache, \
      MODE_CHECK, MODE_ANYTHING, MODE_WRITE, MODE_READ, \
      NeedMoreDataError, UnrecoverableFileError, UncoordinatedWriteError, \
-     NotEnoughServersError
+     NotEnoughServersError, CorruptShareError
 from allmydata.mutable.retrieve import Retrieve
 from allmydata.mutable.publish import Publish
 from allmydata.mutable.servermap import ServerMap, ServermapUpdater
@@ -223,7 +224,7 @@ def flip_bit(original, byte_offset):
             chr(ord(original[byte_offset]) ^ 0x01) +
             original[byte_offset+1:])
 
-def corrupt(res, s, offset, shnums_to_corrupt=None):
+def corrupt(res, s, offset, shnums_to_corrupt=None, offset_offset=0):
     # if shnums_to_corrupt is None, corrupt all shares. Otherwise it is a
     # list of shnums to corrupt.
     for peerid in s._peers:
@@ -250,7 +251,7 @@ def corrupt(res, s, offset, shnums_to_corrupt=None):
                 real_offset = o[offset1]
             else:
                 real_offset = offset1
-            real_offset = int(real_offset) + offset2
+            real_offset = int(real_offset) + offset2 + offset_offset
             assert isinstance(real_offset, int), offset
             shares[shnum] = flip_bit(data, real_offset)
     return res
@@ -646,7 +647,7 @@ class Servermap(unittest.TestCase):
         self.failIfEqual(best, None)
         self.failUnlessEqual(sm.recoverable_versions(), set([best]))
         self.failUnlessEqual(len(sm.shares_available()), 1)
-        self.failUnlessEqual(sm.shares_available()[best], (num_shares, 3))
+        self.failUnlessEqual(sm.shares_available()[best], (num_shares, 3, 10))
         shnum, peerids = sm.make_sharemap().items()[0]
         peerid = list(peerids)[0]
         self.failUnlessEqual(sm.version_on_peer(peerid, shnum), best)
@@ -722,7 +723,7 @@ class Servermap(unittest.TestCase):
             for (shnum, peerid, timestamp) in shares:
                 if shnum < 5:
                     self._corrupted.add( (peerid, shnum) )
-                    sm.mark_bad_share(peerid, shnum)
+                    sm.mark_bad_share(peerid, shnum, "")
             return self.update_servermap(sm, MODE_WRITE)
         d.addCallback(_made_map)
         def _check_map(sm):
@@ -770,7 +771,7 @@ class Servermap(unittest.TestCase):
         best = sm.best_recoverable_version()
         self.failUnlessEqual(best, None)
         self.failUnlessEqual(len(sm.shares_available()), 1)
-        self.failUnlessEqual(sm.shares_available().values()[0], (2,3) )
+        self.failUnlessEqual(sm.shares_available().values()[0], (2,3,10) )
         return sm
 
     def test_not_quite_enough_shares(self):
@@ -806,7 +807,7 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
     def setUp(self):
         # publish a file and create shares, which can then be manipulated
         # later.
-        self.CONTENTS = "New contents go here"
+        self.CONTENTS = "New contents go here" * 1000
         num_peers = 20
         self._client = FakeClient(num_peers)
         self._storage = self._client._storage
@@ -878,9 +879,58 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         d.addCallback(_retrieved)
         return d
 
+    def test_all_shares_vanished(self):
+        d = self.make_servermap()
+        def _remove_shares(servermap):
+            for shares in self._storage._peers.values():
+                shares.clear()
+            d1 = self.shouldFail(NotEnoughSharesError,
+                                 "test_all_shares_vanished",
+                                 "ran out of peers",
+                                 self.do_download, servermap)
+            return d1
+        d.addCallback(_remove_shares)
+        return d
+
+    def test_no_servers(self):
+        c2 = FakeClient(0)
+        self._fn._client = c2
+        # if there are no servers, then a MODE_READ servermap should come
+        # back empty
+        d = self.make_servermap()
+        def _check_servermap(servermap):
+            self.failUnlessEqual(servermap.best_recoverable_version(), None)
+            self.failIf(servermap.recoverable_versions())
+            self.failIf(servermap.unrecoverable_versions())
+            self.failIf(servermap.all_peers())
+        d.addCallback(_check_servermap)
+        return d
+    test_no_servers.timeout = 15
+
+    def test_no_servers_download(self):
+        c2 = FakeClient(0)
+        self._fn._client = c2
+        d = self.shouldFail(UnrecoverableFileError,
+                            "test_no_servers_download",
+                            "no recoverable versions",
+                            self._fn.download_best_version)
+        def _restore(res):
+            # a failed download that occurs while we aren't connected to
+            # anybody should not prevent a subsequent download from working.
+            # This isn't quite the webapi-driven test that #463 wants, but it
+            # should be close enough.
+            self._fn._client = self._client
+            return self._fn.download_best_version()
+        def _retrieved(new_contents):
+            self.failUnlessEqual(new_contents, self.CONTENTS)
+        d.addCallback(_restore)
+        d.addCallback(_retrieved)
+        return d
+    test_no_servers_download.timeout = 15
 
     def _test_corrupt_all(self, offset, substring,
-                          should_succeed=False, corrupt_early=True):
+                          should_succeed=False, corrupt_early=True,
+                          failure_checker=None):
         d = defer.succeed(None)
         if corrupt_early:
             d.addCallback(corrupt, self._storage, offset)
@@ -897,14 +947,16 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
                     self.failUnless(substring in "".join(allproblems))
                 return servermap
             if should_succeed:
-                d1 = self._fn.download_best_version()
+                d1 = self._fn.download_version(servermap, ver)
                 d1.addCallback(lambda new_contents:
                                self.failUnlessEqual(new_contents, self.CONTENTS))
             else:
                 d1 = self.shouldFail(NotEnoughSharesError,
                                      "_corrupt_all(offset=%s)" % (offset,),
                                      substring,
-                                     self._fn.download_best_version)
+                                     self._fn.download_version, servermap, ver)
+            if failure_checker:
+                d1.addCallback(failure_checker)
             d1.addCallback(lambda res: servermap)
             return d1
         d.addCallback(_do_retrieve)
@@ -987,6 +1039,37 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         # writer.
         return self._test_corrupt_all("enc_privkey", None, should_succeed=True)
 
+
+    def test_corrupt_all_seqnum_late(self):
+        # corrupting the seqnum between mapupdate and retrieve should result
+        # in NotEnoughSharesError, since each share will look invalid
+        def _check(res):
+            f = res[0]
+            self.failUnless(f.check(NotEnoughSharesError))
+            self.failUnless("someone wrote to the data since we read the servermap" in str(f))
+        return self._test_corrupt_all(1, "ran out of peers",
+                                      corrupt_early=False,
+                                      failure_checker=_check)
+
+    def test_corrupt_all_block_hash_tree_late(self):
+        def _check(res):
+            f = res[0]
+            self.failUnless(f.check(NotEnoughSharesError))
+        return self._test_corrupt_all("block_hash_tree",
+                                      "block hash tree failure",
+                                      corrupt_early=False,
+                                      failure_checker=_check)
+
+
+    def test_corrupt_all_block_late(self):
+        def _check(res):
+            f = res[0]
+            self.failUnless(f.check(NotEnoughSharesError))
+        return self._test_corrupt_all("share_data", "block hash tree failure",
+                                      corrupt_early=False,
+                                      failure_checker=_check)
+
+
     def test_basic_pubkey_at_end(self):
         # we corrupt the pubkey in all but the last 'k' shares, allowing the
         # download to succeed but forcing a bunch of retries first. Note that
@@ -1037,6 +1120,131 @@ class Roundtrip(unittest.TestCase, testutil.ShouldFailMixin):
         return d
 
 
+class CheckerMixin:
+    def check_good(self, r, where):
+        self.failUnless(r.healthy, where)
+        self.failIf(r.problems, where)
+        return r
+
+    def check_bad(self, r, where):
+        self.failIf(r.healthy, where)
+        return r
+
+    def check_expected_failure(self, r, expected_exception, substring, where):
+        for (peerid, storage_index, shnum, f) in r.problems:
+            if f.check(expected_exception):
+                self.failUnless(substring in str(f),
+                                "%s: substring '%s' not in '%s'" %
+                                (where, substring, str(f)))
+                return
+        self.fail("%s: didn't see expected exception %s in problems %s" %
+                  (where, expected_exception, r.problems))
+
+
+class Checker(unittest.TestCase, CheckerMixin):
+    def setUp(self):
+        # publish a file and create shares, which can then be manipulated
+        # later.
+        self.CONTENTS = "New contents go here" * 1000
+        num_peers = 20
+        self._client = FakeClient(num_peers)
+        self._storage = self._client._storage
+        d = self._client.create_mutable_file(self.CONTENTS)
+        def _created(node):
+            self._fn = node
+        d.addCallback(_created)
+        return d
+
+
+    def test_check_good(self):
+        d = self._fn.check()
+        d.addCallback(self.check_good, "test_check_good")
+        return d
+
+    def test_check_no_shares(self):
+        for shares in self._storage._peers.values():
+            shares.clear()
+        d = self._fn.check()
+        d.addCallback(self.check_bad, "test_check_no_shares")
+        return d
+
+    def test_check_not_enough_shares(self):
+        for shares in self._storage._peers.values():
+            for shnum in shares.keys():
+                if shnum > 0:
+                    del shares[shnum]
+        d = self._fn.check()
+        d.addCallback(self.check_bad, "test_check_not_enough_shares")
+        return d
+
+    def test_check_all_bad_sig(self):
+        corrupt(None, self._storage, 1) # bad sig
+        d = self._fn.check()
+        d.addCallback(self.check_bad, "test_check_all_bad_sig")
+        return d
+
+    def test_check_all_bad_blocks(self):
+        corrupt(None, self._storage, "share_data", [9]) # bad blocks
+        # the Checker won't notice this.. it doesn't look at actual data
+        d = self._fn.check()
+        d.addCallback(self.check_good, "test_check_all_bad_blocks")
+        return d
+
+    def test_verify_good(self):
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_good, "test_verify_good")
+        return d
+
+    def test_verify_all_bad_sig(self):
+        corrupt(None, self._storage, 1) # bad sig
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_bad, "test_verify_all_bad_sig")
+        return d
+
+    def test_verify_one_bad_sig(self):
+        corrupt(None, self._storage, 1, [9]) # bad sig
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_bad, "test_verify_one_bad_sig")
+        return d
+
+    def test_verify_one_bad_block(self):
+        corrupt(None, self._storage, "share_data", [9]) # bad blocks
+        # the Verifier *will* notice this, since it examines every byte
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_bad, "test_verify_one_bad_block")
+        d.addCallback(self.check_expected_failure,
+                      CorruptShareError, "block hash tree failure",
+                      "test_verify_one_bad_block")
+        return d
+
+    def test_verify_one_bad_sharehash(self):
+        corrupt(None, self._storage, "share_hash_chain", [9], 5)
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_bad, "test_verify_one_bad_sharehash")
+        d.addCallback(self.check_expected_failure,
+                      CorruptShareError, "corrupt hashes",
+                      "test_verify_one_bad_sharehash")
+        return d
+
+    def test_verify_one_bad_encprivkey(self):
+        corrupt(None, self._storage, "enc_privkey", [9]) # bad privkey
+        d = self._fn.check(verify=True)
+        d.addCallback(self.check_bad, "test_verify_one_bad_encprivkey")
+        d.addCallback(self.check_expected_failure,
+                      CorruptShareError, "invalid privkey",
+                      "test_verify_one_bad_encprivkey")
+        return d
+
+    def test_verify_one_bad_encprivkey_uncheckable(self):
+        corrupt(None, self._storage, "enc_privkey", [9]) # bad privkey
+        readonly_fn = self._fn.get_readonly()
+        # a read-only node has no way to validate the privkey
+        d = readonly_fn.check(verify=True)
+        d.addCallback(self.check_good,
+                      "test_verify_one_bad_encprivkey_uncheckable")
+        return d
+
+
 class MultipleEncodings(unittest.TestCase):
     def setUp(self):
         self.CONTENTS = "New contents go here"
@@ -1179,7 +1387,7 @@ class MultipleEncodings(unittest.TestCase):
         d.addCallback(_retrieved)
         return d
 
-class MultipleVersions(unittest.TestCase):
+class MultipleVersions(unittest.TestCase, CheckerMixin):
     def setUp(self):
         self.CONTENTS = ["Contents 0",
                          "Contents 1",
@@ -1242,6 +1450,10 @@ class MultipleVersions(unittest.TestCase):
         self._set_versions(dict([(i,2) for i in (0,2,4,6,8)]))
         d = self._fn.download_best_version()
         d.addCallback(lambda res: self.failUnlessEqual(res, self.CONTENTS[4]))
+        # and the checker should report problems
+        d.addCallback(lambda res: self._fn.check())
+        d.addCallback(self.check_bad, "test_multiple_versions")
+
         # but if everything is at version 2, that's what we should download
         d.addCallback(lambda res:
                       self._set_versions(dict([(i,2) for i in range(10)])))
@@ -1580,6 +1792,18 @@ class Problems(unittest.TestCase, testutil.ShouldFailMixin):
                             self.client.create_mutable_file, "contents")
         return d
 
+    def test_publish_no_servers(self):
+        # no servers at all: the publish should fail
+        basedir = os.path.join("mutable/CollidingWrites/publish_no_servers")
+        self.client = LessFakeClient(basedir, 0)
+        d = self.shouldFail(NotEnoughServersError,
+                            "test_publish_no_servers",
+                            "Ran out of non-bad servers",
+                            self.client.create_mutable_file, "contents")
+        return d
+    test_publish_no_servers.timeout = 30
+
+
     def test_privkey_query_error(self):
         # when a servermap is updated with MODE_WRITE, it tries to get the
         # privkey. Something might go wrong during this query attempt.
