@@ -8,14 +8,31 @@ from allmydata import interfaces
 from allmydata.util import fileutil, hashutil
 from allmydata.storage import BucketWriter, BucketReader, \
      WriteBucketProxy, ReadBucketProxy, StorageServer, MutableShareFile, \
-     storage_index_to_dir
+     storage_index_to_dir, DataTooLargeError, LeaseInfo
 from allmydata.interfaces import BadWriteEnablerError
 from allmydata.test.common import LoggingServiceParent
 
+class Marker:
+    pass
 class FakeCanary:
-    def notifyOnDisconnect(self, *args, **kwargs):
-        pass
+    def __init__(self, ignore_disconnectors=False):
+        self.ignore = ignore_disconnectors
+        self.disconnectors = {}
+    def notifyOnDisconnect(self, f, *args, **kwargs):
+        if self.ignore:
+            return
+        m = Marker()
+        self.disconnectors[m] = (f, args, kwargs)
+        return m
     def dontNotifyOnDisconnect(self, marker):
+        if self.ignore:
+            return
+        del self.disconnectors[marker]
+
+class FakeStatsProvider:
+    def count(self, name, delta=1):
+        pass
+    def register_producer(self, producer):
         pass
 
 class Bucket(unittest.TestCase):
@@ -29,13 +46,18 @@ class Bucket(unittest.TestCase):
 
     def bucket_writer_closed(self, bw, consumed):
         pass
+    def add_latency(self, category, latency):
+        pass
+    def count(self, name, delta=1):
+        pass
 
     def make_lease(self):
         owner_num = 0
         renew_secret = os.urandom(32)
         cancel_secret = os.urandom(32)
         expiration_time = time.time() + 5000
-        return (owner_num, renew_secret, cancel_secret, expiration_time)
+        return LeaseInfo(owner_num, renew_secret, cancel_secret,
+                         expiration_time, "\x00" * 20)
 
     def test_create(self):
         incoming, final = self.make_workdir("test_create")
@@ -57,7 +79,7 @@ class Bucket(unittest.TestCase):
         bw.remote_close()
 
         # now read from it
-        br = BucketReader(bw.finalhome)
+        br = BucketReader(self, bw.finalhome)
         self.failUnlessEqual(br.remote_read(0, 25), "a"*25)
         self.failUnlessEqual(br.remote_read(25, 25), "b"*25)
         self.failUnlessEqual(br.remote_read(50, 7), "c"*7)
@@ -88,10 +110,15 @@ class BucketProxy(unittest.TestCase):
         renew_secret = os.urandom(32)
         cancel_secret = os.urandom(32)
         expiration_time = time.time() + 5000
-        return (owner_num, renew_secret, cancel_secret, expiration_time)
+        return LeaseInfo(owner_num, renew_secret, cancel_secret,
+                         expiration_time, "\x00" * 20)
 
     def bucket_writer_closed(self, bw, consumed):
         pass
+    def add_latency(self, category, latency):
+        pass
+    def count(self, name, delta=1):
+        pass
 
     def test_create(self):
         bw, rb, sharefname = self.make_bucket("test_create", 500)
@@ -147,13 +174,15 @@ class BucketProxy(unittest.TestCase):
 
         # now read everything back
         def _start_reading(res):
-            br = BucketReader(sharefname)
+            br = BucketReader(self, sharefname)
             rb = RemoteBucket()
             rb.target = br
-            rbp = ReadBucketProxy(rb)
+            rbp = ReadBucketProxy(rb, peerid="abc")
+            self.failUnless("to peer" in repr(rbp))
             self.failUnless(interfaces.IStorageBucketReader.providedBy(rbp))
 
             d1 = rbp.startIfNecessary()
+            d1.addCallback(lambda res: rbp.startIfNecessary()) # idempotent
             d1.addCallback(lambda res: rbp.get_block(0))
             d1.addCallback(lambda res: self.failUnlessEqual(res, "a"*25))
             d1.addCallback(lambda res: rbp.get_block(1))
@@ -199,19 +228,23 @@ class Server(unittest.TestCase):
 
     def create(self, name, sizelimit=None):
         workdir = self.workdir(name)
-        ss = StorageServer(workdir, sizelimit)
+        ss = StorageServer(workdir, sizelimit,
+                           stats_provider=FakeStatsProvider())
+        ss.setNodeID("\x00" * 20)
         ss.setServiceParent(self.sparent)
         return ss
 
     def test_create(self):
         ss = self.create("test_create")
 
-    def allocate(self, ss, storage_index, sharenums, size):
+    def allocate(self, ss, storage_index, sharenums, size, canary=None):
         renew_secret = hashutil.tagged_hash("blah", "%d" % self._lease_secret.next())
         cancel_secret = hashutil.tagged_hash("blah", "%d" % self._lease_secret.next())
+        if not canary:
+            canary = FakeCanary()
         return ss.remote_allocate_buckets(storage_index,
                                           renew_secret, cancel_secret,
-                                          sharenums, size, FakeCanary())
+                                          sharenums, size, canary)
 
     def test_dont_overfill_dirs(self):
         """
@@ -245,62 +278,97 @@ class Server(unittest.TestCase):
         for i,wb in writers.items():
             wb.remote_write(0, "%10d" % i)
             wb.remote_close()
-        incomingdir = os.path.dirname(os.path.dirname(os.path.dirname(wb.incominghome)))
-        self.failIf(os.path.exists(incomingdir))
+        incoming_share_dir = wb.incominghome
+        incoming_bucket_dir = os.path.dirname(incoming_share_dir)
+        incoming_prefix_dir = os.path.dirname(incoming_bucket_dir)
+        incoming_dir = os.path.dirname(incoming_prefix_dir)
+        self.failIf(os.path.exists(incoming_bucket_dir))
+        self.failIf(os.path.exists(incoming_prefix_dir))
+        self.failUnless(os.path.exists(incoming_dir))
 
     def test_allocate(self):
         ss = self.create("test_allocate")
 
-        self.failUnlessEqual(ss.remote_get_buckets("vid"), {})
+        self.failUnlessEqual(ss.remote_get_buckets("allocate"), {})
 
         canary = FakeCanary()
-        already,writers = self.allocate(ss, "vid", [0,1,2], 75)
+        already,writers = self.allocate(ss, "allocate", [0,1,2], 75)
         self.failUnlessEqual(already, set())
         self.failUnlessEqual(set(writers.keys()), set([0,1,2]))
 
         # while the buckets are open, they should not count as readable
-        self.failUnlessEqual(ss.remote_get_buckets("vid"), {})
+        self.failUnlessEqual(ss.remote_get_buckets("allocate"), {})
 
         # close the buckets
         for i,wb in writers.items():
             wb.remote_write(0, "%25d" % i)
             wb.remote_close()
+            # aborting a bucket that was already closed is a no-op
+            wb.remote_abort()
 
         # now they should be readable
-        b = ss.remote_get_buckets("vid")
+        b = ss.remote_get_buckets("allocate")
         self.failUnlessEqual(set(b.keys()), set([0,1,2]))
         self.failUnlessEqual(b[0].remote_read(0, 25), "%25d" % 0)
 
-        # now if we about writing again, the server should offer those three
-        # buckets as already present. It should offer them even if we don't
-        # ask about those specific ones.
-        already,writers = self.allocate(ss, "vid", [2,3,4], 75)
+        # now if we ask about writing again, the server should offer those
+        # three buckets as already present. It should offer them even if we
+        # don't ask about those specific ones.
+        already,writers = self.allocate(ss, "allocate", [2,3,4], 75)
         self.failUnlessEqual(already, set([0,1,2]))
         self.failUnlessEqual(set(writers.keys()), set([3,4]))
 
         # while those two buckets are open for writing, the server should
         # refuse to offer them to uploaders
 
-        already,writers = self.allocate(ss, "vid", [2,3,4,5], 75)
-        self.failUnlessEqual(already, set([0,1,2]))
-        self.failUnlessEqual(set(writers.keys()), set([5]))
+        already2,writers2 = self.allocate(ss, "allocate", [2,3,4,5], 75)
+        self.failUnlessEqual(already2, set([0,1,2]))
+        self.failUnlessEqual(set(writers2.keys()), set([5]))
+
+        # aborting the writes should remove the tempfiles
+        for i,wb in writers2.items():
+            wb.remote_abort()
+        already2,writers2 = self.allocate(ss, "allocate", [2,3,4,5], 75)
+        self.failUnlessEqual(already2, set([0,1,2]))
+        self.failUnlessEqual(set(writers2.keys()), set([5]))
+
+        for i,wb in writers2.items():
+            wb.remote_abort()
+        for i,wb in writers.items():
+            wb.remote_abort()
+
+    def test_disconnect(self):
+        # simulate a disconnection
+        ss = self.create("test_disconnect")
+        canary = FakeCanary()
+        already,writers = self.allocate(ss, "disconnect", [0,1,2], 75, canary)
+        self.failUnlessEqual(already, set())
+        self.failUnlessEqual(set(writers.keys()), set([0,1,2]))
+        for (f,args,kwargs) in canary.disconnectors.values():
+            f(*args, **kwargs)
+        del already
+        del writers
+
+        # that ought to delete the incoming shares
+        already,writers = self.allocate(ss, "disconnect", [0,1,2], 75)
+        self.failUnlessEqual(already, set())
+        self.failUnlessEqual(set(writers.keys()), set([0,1,2]))
 
     def test_sizelimits(self):
         ss = self.create("test_sizelimits", 5000)
-        canary = FakeCanary()
         # a newly created and filled share incurs this much overhead, beyond
         # the size we request.
         OVERHEAD = 3*4
         LEASE_SIZE = 4+32+32+4
-
-        already,writers = self.allocate(ss, "vid1", [0,1,2], 1000)
+        canary = FakeCanary(True)
+        already,writers = self.allocate(ss, "vid1", [0,1,2], 1000, canary)
         self.failUnlessEqual(len(writers), 3)
         # now the StorageServer should have 3000 bytes provisionally
         # allocated, allowing only 2000 more to be claimed
         self.failUnlessEqual(len(ss._active_writers), 3)
 
         # allocating 1001-byte shares only leaves room for one
-        already2,writers2 = self.allocate(ss, "vid2", [0,1,2], 1001)
+        already2,writers2 = self.allocate(ss, "vid2", [0,1,2], 1001, canary)
         self.failUnlessEqual(len(writers2), 1)
         self.failUnlessEqual(len(ss._active_writers), 4)
 
@@ -325,7 +393,7 @@ class Server(unittest.TestCase):
         allocated = 1001 + OVERHEAD + LEASE_SIZE
         # now there should be ALLOCATED=1001+12+72=1085 bytes allocated, and
         # 5000-1085=3915 free, therefore we can fit 39 100byte shares
-        already3,writers3 = self.allocate(ss,"vid3", range(100), 100)
+        already3,writers3 = self.allocate(ss,"vid3", range(100), 100, canary)
         self.failUnlessEqual(len(writers3), 39)
         self.failUnlessEqual(len(ss._active_writers), 39)
 
@@ -344,7 +412,7 @@ class Server(unittest.TestCase):
         # extra-file metadata, the allocation would be more than 'allocated'
         # and this test would need to be changed.
         ss = self.create("test_sizelimits", 5000)
-        already4,writers4 = self.allocate(ss, "vid4", range(100), 100)
+        already4,writers4 = self.allocate(ss, "vid4", range(100), 100, canary)
         self.failUnlessEqual(len(writers4), 39)
         self.failUnlessEqual(len(ss._active_writers), 39)
 
@@ -385,7 +453,7 @@ class Server(unittest.TestCase):
 
         leases = list(ss.get_leases("si0"))
         self.failUnlessEqual(len(leases), 1)
-        self.failUnlessEqual(set([l[1] for l in leases]), set([rs0]))
+        self.failUnlessEqual(set([l.renew_secret for l in leases]), set([rs0]))
 
         rs1,cs1 = (hashutil.tagged_hash("blah", "%d" % self._lease_secret.next()),
                    hashutil.tagged_hash("blah", "%d" % self._lease_secret.next()))
@@ -404,7 +472,7 @@ class Server(unittest.TestCase):
 
         leases = list(ss.get_leases("si1"))
         self.failUnlessEqual(len(leases), 2)
-        self.failUnlessEqual(set([l[1] for l in leases]), set([rs1, rs2]))
+        self.failUnlessEqual(set([l.renew_secret for l in leases]), set([rs1, rs2]))
 
         # check that si0 is readable
         readers = ss.remote_get_buckets("si0")
@@ -440,7 +508,7 @@ class Server(unittest.TestCase):
 
         leases = list(ss.get_leases("si1"))
         self.failUnlessEqual(len(leases), 1)
-        self.failUnlessEqual(set([l[1] for l in leases]), set([rs2]))
+        self.failUnlessEqual(set([l.renew_secret for l in leases]), set([rs2]))
 
         ss.remote_renew_lease("si1", rs2)
         # cancelling the second should make it go away
@@ -481,6 +549,38 @@ class Server(unittest.TestCase):
         leases = list(ss.get_leases("si3"))
         self.failUnlessEqual(len(leases), 2)
 
+    def test_readonly(self):
+        workdir = self.workdir("test_readonly")
+        ss = StorageServer(workdir, readonly_storage=True)
+        ss.setNodeID("\x00" * 20)
+        ss.setServiceParent(self.sparent)
+
+        canary = FakeCanary()
+        already,writers = self.allocate(ss, "vid", [0,1,2], 75)
+        self.failUnlessEqual(already, set())
+        self.failUnlessEqual(writers, {})
+
+    def test_discard(self):
+        # discard is really only used for other tests, but we test it anyways
+        workdir = self.workdir("test_discard")
+        ss = StorageServer(workdir, discard_storage=True)
+        ss.setNodeID("\x00" * 20)
+        ss.setServiceParent(self.sparent)
+
+        canary = FakeCanary()
+        already,writers = self.allocate(ss, "vid", [0,1,2], 75)
+        self.failUnlessEqual(already, set())
+        self.failUnlessEqual(set(writers.keys()), set([0,1,2]))
+        for i,wb in writers.items():
+            wb.remote_write(0, "%25d" % i)
+            wb.remote_close()
+        # since we discard the data, the shares should be present but sparse.
+        # Since we write with some seeks, the data we read back will be all
+        # zeros.
+        b = ss.remote_get_buckets("vid")
+        self.failUnlessEqual(set(b.keys()), set([0,1,2]))
+        self.failUnlessEqual(b[0].remote_read(0, 25), "\x00" * 25)
+
 
 
 class MutableServer(unittest.TestCase):
@@ -499,7 +599,7 @@ class MutableServer(unittest.TestCase):
         workdir = self.workdir(name)
         ss = StorageServer(workdir, sizelimit)
         ss.setServiceParent(self.sparent)
-        ss.setNodeID("\x00" * 32)
+        ss.setNodeID("\x00" * 20)
         return ss
 
     def test_create(self):
@@ -531,10 +631,38 @@ class MutableServer(unittest.TestCase):
         self.failUnless(isinstance(readv_data, dict))
         self.failUnlessEqual(len(readv_data), 0)
 
+    def test_container_size(self):
+        ss = self.create("test_container_size")
+        self.allocate(ss, "si1", "we1", self._lease_secret.next(),
+                      set([0,1,2]), 100)
+        rstaraw = ss.remote_slot_testv_and_readv_and_writev
+        secrets = ( self.write_enabler("we1"),
+                    self.renew_secret("we1"),
+                    self.cancel_secret("we1") )
+        data = "".join([ ("%d" % i) * 10 for i in range(10) ])
+        answer = rstaraw("si1", secrets,
+                         {0: ([], [(0,data)], len(data)+12)},
+                         [])
+        self.failUnlessEqual(answer, (True, {0:[],1:[],2:[]}) )
+
+        # trying to make the container too large will raise an exception
+        TOOBIG = MutableShareFile.MAX_SIZE + 10
+        self.failUnlessRaises(DataTooLargeError,
+                              rstaraw, "si1", secrets,
+                              {0: ([], [(0,data)], TOOBIG)},
+                              [])
+
+        # it should be possible to make the container smaller, although at
+        # the moment this doesn't actually affect the share
+        answer = rstaraw("si1", secrets,
+                         {0: ([], [(0,data)], len(data)+8)},
+                         [])
+        self.failUnlessEqual(answer, (True, {0:[],1:[],2:[]}) )
+
     def test_allocate(self):
         ss = self.create("test_allocate")
         self.allocate(ss, "si1", "we1", self._lease_secret.next(),
-                               set([0,1,2]), 100)
+                      set([0,1,2]), 100)
 
         read = ss.remote_slot_readv
         self.failUnlessEqual(read("si1", [0], [(0, 10)]),
@@ -771,6 +899,16 @@ class MutableServer(unittest.TestCase):
         self.failUnlessEqual(read("si1", [0], [(0,100)]), {0: [data]})
         reset()
 
+        # finally, test some operators against empty shares
+        answer = write("si1", secrets, {1: ([(10, 5, "eq", "11112"),
+                                             ],
+                                            [(0, "x"*100)],
+                                            None,
+                                            )}, [(10,5)])
+        self.failUnlessEqual(answer, (False, {0: ["11111"]}))
+        self.failUnlessEqual(read("si1", [0], [(0,100)]), {0: [data]})
+        reset()
+
     def test_readv(self):
         ss = self.create("test_readv")
         secrets = ( self.write_enabler("we1"),
@@ -792,20 +930,31 @@ class MutableServer(unittest.TestCase):
                                       1: ["1"*10],
                                       2: ["2"*10]})
 
-    def compare_leases_without_timestamps(self, a, b):
-        self.failUnlessEqual(len(a), len(b))
-        for i in range(len(a)):
-            (num_a, (ownerid_a, expiration_time_a,
-                   renew_secret_a, cancel_secret_a, nodeid_a)) = a[i]
-            (num_b, (ownerid_b, expiration_time_b,
-                   renew_secret_b, cancel_secret_b, nodeid_b)) = b[i]
-            self.failUnlessEqual( (num_a, ownerid_a, renew_secret_a,
-                                   cancel_secret_a, nodeid_a),
-                                  (num_b, ownerid_b, renew_secret_b,
-                                   cancel_secret_b, nodeid_b) )
+    def compare_leases_without_timestamps(self, leases_a, leases_b):
+        self.failUnlessEqual(len(leases_a), len(leases_b))
+        for i in range(len(leases_a)):
+            num_a, a = leases_a[i]
+            num_b, b = leases_b[i]
+            self.failUnlessEqual(num_a, num_b)
+            self.failUnlessEqual(a.owner_num,       b.owner_num)
+            self.failUnlessEqual(a.renew_secret,    b.renew_secret)
+            self.failUnlessEqual(a.cancel_secret,   b.cancel_secret)
+            self.failUnlessEqual(a.nodeid,          b.nodeid)
+
+    def compare_leases(self, leases_a, leases_b):
+        self.failUnlessEqual(len(leases_a), len(leases_b))
+        for i in range(len(leases_a)):
+            num_a, a = leases_a[i]
+            num_b, b = leases_b[i]
+            self.failUnlessEqual(num_a, num_b)
+            self.failUnlessEqual(a.owner_num,       b.owner_num)
+            self.failUnlessEqual(a.renew_secret,    b.renew_secret)
+            self.failUnlessEqual(a.cancel_secret,   b.cancel_secret)
+            self.failUnlessEqual(a.nodeid,          b.nodeid)
+            self.failUnlessEqual(a.expiration_time, b.expiration_time)
 
     def test_leases(self):
-        ss = self.create("test_leases")
+        ss = self.create("test_leases", sizelimit=1000*1000)
         def secrets(n):
             return ( self.write_enabler("we1"),
                      self.renew_secret("we1-%d" % n),
@@ -869,20 +1018,25 @@ class MutableServer(unittest.TestCase):
         all_leases = s0.debug_get_leases()
         # renewing with a bogus token should prompt an error message
 
-        # TODO: examine the exception thus raised, make sure the old nodeid
-        # is present, to provide for share migration
-        self.failUnlessRaises(IndexError,
-                              ss.remote_renew_lease, "si1",
-                              secrets(20)[1])
+        # examine the exception thus raised, make sure the old nodeid is
+        # present, to provide for share migration
+        e = self.failUnlessRaises(IndexError,
+                                  ss.remote_renew_lease, "si1",
+                                  secrets(20)[1])
+        e_s = str(e)
+        self.failUnless("Unable to renew non-existent lease" in e_s)
+        self.failUnless("I have leases accepted by nodeids:" in e_s)
+        self.failUnless("nodeids: 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa' ." in e_s)
+
         # same for cancelling
         self.failUnlessRaises(IndexError,
                               ss.remote_cancel_lease, "si1",
                               secrets(20)[2])
-        self.failUnlessEqual(all_leases, s0.debug_get_leases())
+        self.compare_leases(all_leases, s0.debug_get_leases())
 
         # reading shares should not modify the timestamp
         read("si1", [], [(0,200)])
-        self.failUnlessEqual(all_leases, s0.debug_get_leases())
+        self.compare_leases(all_leases, s0.debug_get_leases())
 
         write("si1", secrets(0),
               {0: ([], [(200, "make me bigger")], None)}, [])
@@ -905,8 +1059,94 @@ class MutableServer(unittest.TestCase):
         self.failUnlessEqual(len(remaining_shares), 1)
         self.failUnlessEqual(len(s0.debug_get_leases()), 1)
 
+        # cancelling a non-existent lease should raise an IndexError
+        self.failUnlessRaises(IndexError,
+                              ss.remote_cancel_lease, "si1", "nonsecret")
+
+        # and the slot should still be there
+        remaining_shares = read("si1", [], [(0,10)])
+        self.failUnlessEqual(len(remaining_shares), 1)
+        self.failUnlessEqual(len(s0.debug_get_leases()), 1)
+
         ss.remote_cancel_lease("si1", secrets(4)[2])
         # now the slot should be gone
         no_shares = read("si1", [], [(0,10)])
         self.failUnlessEqual(no_shares, {})
 
+        # cancelling a lease on a non-existent share should raise an IndexError
+        self.failUnlessRaises(IndexError,
+                              ss.remote_cancel_lease, "si2", "nonsecret")
+
+
+class Stats(unittest.TestCase):
+
+    def setUp(self):
+        self.sparent = LoggingServiceParent()
+        self._lease_secret = itertools.count()
+    def tearDown(self):
+        return self.sparent.stopService()
+
+    def workdir(self, name):
+        basedir = os.path.join("storage", "Server", name)
+        return basedir
+
+    def create(self, name, sizelimit=None):
+        workdir = self.workdir(name)
+        ss = StorageServer(workdir, sizelimit)
+        ss.setNodeID("\x00" * 20)
+        ss.setServiceParent(self.sparent)
+        return ss
+
+    def test_latencies(self):
+        ss = self.create("test_latencies")
+        for i in range(10000):
+            ss.add_latency("allocate", 1.0 * i)
+        for i in range(1000):
+            ss.add_latency("renew", 1.0 * i)
+        for i in range(10):
+            ss.add_latency("cancel", 2.0 * i)
+        ss.add_latency("get", 5.0)
+
+        output = ss.get_latencies()
+
+        self.failUnlessEqual(sorted(output.keys()),
+                             sorted(["allocate", "renew", "cancel", "get"]))
+        self.failUnlessEqual(len(ss.latencies["allocate"]), 1000)
+        self.failUnless(abs(output["allocate"]["mean"] - 9500) < 1)
+        self.failUnless(abs(output["allocate"]["01_0_percentile"] - 9010) < 1)
+        self.failUnless(abs(output["allocate"]["10_0_percentile"] - 9100) < 1)
+        self.failUnless(abs(output["allocate"]["50_0_percentile"] - 9500) < 1)
+        self.failUnless(abs(output["allocate"]["90_0_percentile"] - 9900) < 1)
+        self.failUnless(abs(output["allocate"]["95_0_percentile"] - 9950) < 1)
+        self.failUnless(abs(output["allocate"]["99_0_percentile"] - 9990) < 1)
+        self.failUnless(abs(output["allocate"]["99_9_percentile"] - 9999) < 1)
+
+        self.failUnlessEqual(len(ss.latencies["renew"]), 1000)
+        self.failUnless(abs(output["renew"]["mean"] - 500) < 1)
+        self.failUnless(abs(output["renew"]["01_0_percentile"] -  10) < 1)
+        self.failUnless(abs(output["renew"]["10_0_percentile"] - 100) < 1)
+        self.failUnless(abs(output["renew"]["50_0_percentile"] - 500) < 1)
+        self.failUnless(abs(output["renew"]["90_0_percentile"] - 900) < 1)
+        self.failUnless(abs(output["renew"]["95_0_percentile"] - 950) < 1)
+        self.failUnless(abs(output["renew"]["99_0_percentile"] - 990) < 1)
+        self.failUnless(abs(output["renew"]["99_9_percentile"] - 999) < 1)
+
+        self.failUnlessEqual(len(ss.latencies["cancel"]), 10)
+        self.failUnless(abs(output["cancel"]["mean"] - 9) < 1)
+        self.failUnless(abs(output["cancel"]["01_0_percentile"] -  0) < 1)
+        self.failUnless(abs(output["cancel"]["10_0_percentile"] -  2) < 1)
+        self.failUnless(abs(output["cancel"]["50_0_percentile"] - 10) < 1)
+        self.failUnless(abs(output["cancel"]["90_0_percentile"] - 18) < 1)
+        self.failUnless(abs(output["cancel"]["95_0_percentile"] - 18) < 1)
+        self.failUnless(abs(output["cancel"]["99_0_percentile"] - 18) < 1)
+        self.failUnless(abs(output["cancel"]["99_9_percentile"] - 18) < 1)
+
+        self.failUnlessEqual(len(ss.latencies["get"]), 1)
+        self.failUnless(abs(output["get"]["mean"] - 5) < 1)
+        self.failUnless(abs(output["get"]["01_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["10_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["50_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["90_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["95_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["99_0_percentile"] - 5) < 1)
+        self.failUnless(abs(output["get"]["99_9_percentile"] - 5) < 1)
