@@ -11,7 +11,7 @@ from nevow.inevow import IRequest
 
 from foolscap.eventual import fireEventually
 
-from allmydata.util import log, base32
+from allmydata.util import base32
 from allmydata.uri import from_string_verifier, from_string_dirnode, \
      CHKFileVerifierURI
 from allmydata.interfaces import IDirectoryNode, IFileNode, IMutableFileNode, \
@@ -21,6 +21,7 @@ from allmydata.web.common import text_plain, WebError, IClient, \
      getxmlfile, RenderMixin
 from allmydata.web.filenode import ReplaceMeMixin, \
      FileNodeHandler, PlaceHolderNodeHandler
+from allmydata.web.checker_results import CheckerResults, DeepCheckResults
 
 class BlockingFileError(Exception):
     # TODO: catch and transform
@@ -181,6 +182,8 @@ class DirectoryNodeHandler(RenderMixin, rend.Page, ReplaceMeMixin):
             d = self._POST_rename(req)
         elif t == "check":
             d = self._POST_check(req)
+        elif t == "deep-check":
+            d = self._POST_deep_check(req)
         elif t == "set_children":
             # TODO: docs
             d = self._POST_set_children(req)
@@ -330,12 +333,13 @@ class DirectoryNodeHandler(RenderMixin, rend.Page, ReplaceMeMixin):
     def _POST_check(self, req):
         # check this directory
         d = self.node.check()
-        def _done(res):
-            log.msg("checked %s, results %s" % (self.node, res),
-                    facility="tahoe.webish", level=log.NOISY)
-            return str(res)
-        d.addCallback(_done)
-        # TODO: results
+        d.addCallback(lambda res: CheckerResults(res))
+        return d
+
+    def _POST_deep_check(self, req):
+        # check this directory and everything reachable from it
+        d = self.node.deep_check()
+        d.addCallback(lambda res: DeepCheckResults(res))
         return d
 
     def _POST_set_children(self, req):
@@ -367,7 +371,7 @@ class DirectoryAsHTML(rend.Page):
     # The remainder of this class is to render the directory into
     # human+browser -oriented HTML.
     docFactory = getxmlfile("directory.xhtml")
-    #addSlash = True
+    addSlash = True
 
     def __init__(self, node):
         rend.Page.__init__(self)
@@ -385,9 +389,15 @@ class DirectoryAsHTML(rend.Page):
             header.append(" (readonly)")
         return ctx.tag[header]
 
-    def render_welcome(self, ctx, data):
-        depth = len(IRequest(ctx).path) + 2
+    def get_root(self, ctx):
+        req = IRequest(ctx)
+        # the addSlash=True gives us one extra (empty) segment
+        depth = len(req.prepath) + len(req.postpath) - 1
         link = "/".join([".."] * depth)
+        return link
+
+    def render_welcome(self, ctx, data):
+        link = self.get_root(ctx)
         return T.div[T.a(href=link)["Return to Welcome page"]]
 
     def data_children(self, ctx, data):
@@ -419,6 +429,8 @@ class DirectoryAsHTML(rend.Page):
         name = name.encode("utf-8")
         assert not isinstance(name, unicode)
 
+        root = self.get_root(ctx)
+        here = "%s/uri/%s/" % (root, urllib.quote(self.node.get_uri()))
         if self.node.is_readonly():
             delete = "-"
             rename = "-"
@@ -426,25 +438,31 @@ class DirectoryAsHTML(rend.Page):
             # this creates a button which will cause our child__delete method
             # to be invoked, which deletes the file and then redirects the
             # browser back to this directory
-            delete = T.form(action=url.here, method="post")[
+            delete = T.form(action=here, method="post")[
                 T.input(type='hidden', name='t', value='delete'),
                 T.input(type='hidden', name='name', value=name),
-                T.input(type='hidden', name='when_done', value=url.here),
+                T.input(type='hidden', name='when_done', value="."),
                 T.input(type='submit', value='del', name="del"),
                 ]
 
-            rename = T.form(action=url.here, method="get")[
+            rename = T.form(action=here, method="get")[
                 T.input(type='hidden', name='t', value='rename-form'),
                 T.input(type='hidden', name='name', value=name),
-                T.input(type='hidden', name='when_done', value=url.here),
+                T.input(type='hidden', name='when_done', value="."),
                 T.input(type='submit', value='rename', name="rename"),
                 ]
 
         ctx.fillSlots("delete", delete)
         ctx.fillSlots("rename", rename)
-        check = T.form(action=url.here.child(name), method="post")[
+        if IDirectoryNode.providedBy(target):
+            check_url = "%s/uri/%s/" % (root, urllib.quote(target.get_uri()))
+            check_done_url = "../../uri/%s/" % urllib.quote(self.node.get_uri())
+        else:
+            check_url = "%s/uri/%s" % (root, urllib.quote(target.get_uri()))
+            check_done_url = "../uri/%s/" % urllib.quote(self.node.get_uri())
+        check = T.form(action=check_url, method="post")[
             T.input(type='hidden', name='t', value='check'),
-            T.input(type='hidden', name='when_done', value=url.here),
+            T.input(type='hidden', name='return_to', value=check_done_url),
             T.input(type='submit', value='check', name="check"),
             ]
         ctx.fillSlots("overwrite",
@@ -475,7 +493,7 @@ class DirectoryAsHTML(rend.Page):
             # to prevent javascript in displayed .html files from stealing a
             # secret directory URI from the URL, send the browser to a URI-based
             # page that doesn't know about the directory at all
-            dlurl = "/file/%s/@@named=/%s" % (quoted_uri, urllib.quote(name))
+            dlurl = "%s/file/%s/@@named=/%s" % (root, quoted_uri, urllib.quote(name))
 
             ctx.fillSlots("filename",
                           T.a(href=dlurl)[html.escape(name)])
@@ -483,11 +501,11 @@ class DirectoryAsHTML(rend.Page):
 
             ctx.fillSlots("size", "?")
 
-            text_plain_url = "/file/%s/@@named=/foo.txt" % quoted_uri
+            text_plain_url = "%s/file/%s/@@named=/foo.txt" % (root, quoted_uri)
             text_plain_tag = T.a(href=text_plain_url)["text/plain"]
 
         elif IFileNode.providedBy(target):
-            dlurl = "/file/%s/@@named=/%s" % (quoted_uri, urllib.quote(name))
+            dlurl = "%s/file/%s/@@named=/%s" % (root, quoted_uri, urllib.quote(name))
 
             ctx.fillSlots("filename",
                           T.a(href=dlurl)[html.escape(name)])
@@ -495,13 +513,13 @@ class DirectoryAsHTML(rend.Page):
 
             ctx.fillSlots("size", target.get_size())
 
-            text_plain_url = "/file/%s/@@named=/foo.txt" % quoted_uri
+            text_plain_url = "%s/file/%s/@@named=/foo.txt" % (root, quoted_uri)
             text_plain_tag = T.a(href=text_plain_url)["text/plain"]
 
 
         elif IDirectoryNode.providedBy(target):
             # directory
-            uri_link = "/uri/" + urllib.quote(target.get_uri()) + "/"
+            uri_link = "%s/uri/%s/" % (root, urllib.quote(target.get_uri()))
             ctx.fillSlots("filename",
                           T.a(href=uri_link)[html.escape(name)])
             if target.is_readonly():
@@ -521,34 +539,7 @@ class DirectoryAsHTML(rend.Page):
 
         ctx.fillSlots("data", childdata)
 
-        try:
-            checker = IClient(ctx).getServiceNamed("checker")
-        except KeyError:
-            checker = None
-        if checker:
-            d = defer.maybeDeferred(checker.checker_results_for,
-                                    target.get_verifier())
-            def _got(checker_results):
-                recent_results = reversed(checker_results[-5:])
-                if IFileNode.providedBy(target):
-                    results = ("[" +
-                               ", ".join(["%d/%d" % (found, needed)
-                                          for (when,
-                                               (needed, total, found, sharemap))
-                                          in recent_results]) +
-                               "]")
-                elif IDirectoryNode.providedBy(target):
-                    results = ("[" +
-                               "".join([{True:"+",False:"-"}[res]
-                                        for (when, res) in recent_results]) +
-                               "]")
-                else:
-                    results = "%d results" % len(checker_results)
-                return results
-            d.addCallback(_got)
-            results = d
-        else:
-            results = "--"
+        results = "--"
         # TODO: include a link to see more results, including timestamps
         # TODO: use a sparkline
         ctx.fillSlots("checker_results", results)
@@ -556,24 +547,41 @@ class DirectoryAsHTML(rend.Page):
         return ctx.tag
 
     def render_forms(self, ctx, data):
+        forms = []
+        deep_check = T.form(action=".", method="post",
+                            enctype="multipart/form-data")[
+            T.fieldset[
+            T.input(type="hidden", name="t", value="deep-check"),
+            T.input(type="hidden", name="return_to", value="."),
+            T.legend(class_="freeform-form-label")["Run a deep-check operation (EXPENSIVE)"],
+            T.input(type="submit", value="Deep-Check"),
+            " ",
+            "Verify every bit? (EVEN MORE EXPENSIVE):",
+            T.input(type="checkbox", name="verify"),
+            ]]
+        forms.append(T.div(class_="freeform-form")[deep_check])
+
         if self.node.is_readonly():
-            return T.div["No upload forms: directory is read-only"]
+            forms.append(T.div["No upload forms: directory is read-only"])
+            return forms
+
         mkdir = T.form(action=".", method="post",
                        enctype="multipart/form-data")[
             T.fieldset[
             T.input(type="hidden", name="t", value="mkdir"),
-            T.input(type="hidden", name="when_done", value=url.here),
+            T.input(type="hidden", name="when_done", value="."),
             T.legend(class_="freeform-form-label")["Create a new directory"],
             "New directory name: ",
             T.input(type="text", name="name"), " ",
             T.input(type="submit", value="Create"),
             ]]
+        forms.append(T.div(class_="freeform-form")[mkdir])
 
         upload = T.form(action=".", method="post",
                         enctype="multipart/form-data")[
             T.fieldset[
             T.input(type="hidden", name="t", value="upload"),
-            T.input(type="hidden", name="when_done", value=url.here),
+            T.input(type="hidden", name="when_done", value="."),
             T.legend(class_="freeform-form-label")["Upload a file to this directory"],
             "Choose a file to upload: ",
             T.input(type="file", name="file", class_="freeform-input-file"),
@@ -582,12 +590,13 @@ class DirectoryAsHTML(rend.Page):
             " Mutable?:",
             T.input(type="checkbox", name="mutable"),
             ]]
+        forms.append(T.div(class_="freeform-form")[upload])
 
         mount = T.form(action=".", method="post",
                         enctype="multipart/form-data")[
             T.fieldset[
             T.input(type="hidden", name="t", value="uri"),
-            T.input(type="hidden", name="when_done", value=url.here),
+            T.input(type="hidden", name="when_done", value="."),
             T.legend(class_="freeform-form-label")["Attach a file or directory"
                                                    " (by URI) to this"
                                                    " directory"],
@@ -597,19 +606,19 @@ class DirectoryAsHTML(rend.Page):
             T.input(type="text", name="uri"), " ",
             T.input(type="submit", value="Attach"),
             ]]
-        return [T.div(class_="freeform-form")[mkdir],
-                T.div(class_="freeform-form")[upload],
-                T.div(class_="freeform-form")[mount],
-                ]
+        forms.append(T.div(class_="freeform-form")[mount])
+        return forms
 
     def build_overwrite_form(self, ctx, name, target):
         if IMutableFileNode.providedBy(target) and not target.is_readonly():
-            action = "/uri/" + urllib.quote(target.get_uri())
+            root = self.get_root(ctx)
+            action = "%s/uri/%s" % (root, urllib.quote(target.get_uri()))
+            done_url = "../uri/%s/" % urllib.quote(self.node.get_uri())
             overwrite = T.form(action=action, method="post",
                                enctype="multipart/form-data")[
                 T.fieldset[
                 T.input(type="hidden", name="t", value="upload"),
-                T.input(type='hidden', name='when_done', value=url.here),
+                T.input(type='hidden', name='when_done', value=done_url),
                 T.legend(class_="freeform-form-label")["Overwrite"],
                 "Choose new file: ",
                 T.input(type="file", name="file", class_="freeform-input-file"),
@@ -692,7 +701,7 @@ class RenameForm(rend.Page):
         return ctx.tag[header]
 
     def render_when_done(self, ctx, data):
-        return T.input(type="hidden", name="when_done", value=url.here)
+        return T.input(type="hidden", name="when_done", value=".")
 
     def render_get_name(self, ctx, data):
         req = IRequest(ctx)
